@@ -9,7 +9,9 @@ function(sgp_object,
 	outputSGP_INDIVIDUAL.content_areas=NULL,
 	outputSGP.anonymize=FALSE,
 	outputSGP.student.groups=NULL,
-	outputSGP.directory="Data") {
+	outputSGP.directory="Data",
+	outputSGP.translate.names=TRUE,
+	outputSGP.projection.years.for.target=3) {
 
         started.at.outputSGP <- proc.time()
         message(paste("\nStarted outputSGP ", date(), ": Files produced from outputSGP saved in '", outputSGP.directory, "'\n", sep=""))
@@ -21,16 +23,14 @@ function(sgp_object,
 	### Define varaibles (to prevent R CMD check warnings)
 
 	SCALE_SCORE <- CONTENT_AREA <- YEAR <- GRADE <- ID <- ETHNICITY <- GENDER <- LAST_NAME <- FIRST_NAME <- VALID_CASE <- DISTRICT_NUMBER <- SCHOOL_NUMBER <- YEAR_BY_CONTENT_AREA <- NULL
-	names.type <- names.provided <- names.output <- NULL
+	names.type <- names.provided <- names.output <- STATE_ENROLLMENT_STATUS <- EMH_LEVEL <- NULL
 
 	### Create state (if missing) from sgp_object (if possible)
 
-	if (is.null(state)) {
-		tmp.name <- gsub("_", " ", deparse(substitute(sgp_object)))
-		if (any(sapply(c(state.name, "Demonstration", "AOB"), function(x) regexpr(x, tmp.name))==1)) {
-			state <- c(state.abb, "DEMO", "AOB")[which(sapply(c(state.name, "Demonstration", "AOB"), function(x) regexpr(x, tmp.name))==1)]
-		}
-	}
+        if (is.null(state)) {
+                tmp.name <- toupper(gsub("_", " ", deparse(substitute(sgp_object))))
+                state <- getStateAbbreviation(tmp.name, "outputSGP")
+        }
 
 
 	### Create relevant variables
@@ -63,17 +63,27 @@ function(sgp_object,
 		started.at <- proc.time()
 		message(paste("\tStarted LONG data production in outputSGP", date()))
 
+		names.in.data <- which(sgp_object@Names[['names.sgp']] %in% names(sgp_object@Data))
+		if (outputSGP.translate.names) setnames(sgp_object@Data, sgp_object@Names[['names.sgp']][names.in.data], sgp_object@Names[['names.provided']][names.in.data])
 		write.table(sgp_object@Data, file=file.path(outputSGP.directory, paste(tmp.state, "SGP_LONG_Data.txt", sep="_")), sep="|", quote=FALSE, row.names=FALSE, na="")
 		if (identical(.Platform$OS.type, "unix")) {
 			if (file.info(file.path(outputSGP.directory, paste(tmp.state, "SGP_LONG_Data.txt", sep="_")))$size > 4000000000) {
-				system(paste("gzip", file.path(outputSGP.directory, paste(tmp.state, "SGP_LONG_Data.txt", sep="_"))))
+				tmp.working.directory <- getwd()
+				setwd(file.path(outputSGP.directory))
+				if (paste(tmp.state, "SGP_LONG_Data.txt.gz", sep="_") %in% list.files()) file.remove(paste(tmp.state, "SGP_LONG_Data.txt.gz", sep="_"))
+				system(paste("gzip", paste(tmp.state, "SGP_LONG_Data.txt", sep="_")))
+				setwd(tmp.working.directory)
 			} else {
-				suppressWarnings(
-					zip(file.path(outputSGP.directory, paste(tmp.state, "SGP_LONG_Data.txt.zip", sep="_")), 
-						file.path(outputSGP.directory, paste(tmp.state, "SGP_LONG_Data.txt", sep="_")))
+				tmp.working.directory <- getwd()
+				setwd(file.path(outputSGP.directory))
+				if (paste(tmp.state, "SGP_LONG_Data.txt.zip", sep="_") %in% list.files()) file.remove(paste(tmp.state, "SGP_LONG_Data.txt.zip", sep="_"))
+				suppressMessages(
+					zip(paste(tmp.state, "SGP_LONG_Data.txt.zip", sep="_"), paste(tmp.state, "SGP_LONG_Data.txt", sep="_"))
 				)
+				setwd(tmp.working.directory)
 			}
 		}
+		if (outputSGP.translate.names) setnames(sgp_object@Data, sgp_object@Names[['names.provided']][names.in.data], sgp_object@Names[['names.sgp']][names.in.data])
 
 		message(paste("\tFinished LONG data production in outputSGP", date(), "in", timetaken(started.at), "\n"))
 
@@ -101,11 +111,12 @@ function(sgp_object,
 		started.at <- proc.time()
 		message(paste("\tStarted WIDE data production in outputSGP", date()))
 
-		long_data_tmp <- sgp_object@Data
-		setkeyv(long_data_tmp, c("VALID_CASE", "ID"))
-		suppressWarnings(invisible(long_data_tmp[,YEAR_BY_CONTENT_AREA := paste(YEAR, CONTENT_AREA, sep=".")]))
+		long_data_tmp <- copy(sgp_object@Data)
+		setkeyv(long_data_tmp, c("VALID_CASE", "YEAR", "CONTENT_AREA", "ID"))
+		suppressMessages(invisible(long_data_tmp[,YEAR_BY_CONTENT_AREA := paste(YEAR, CONTENT_AREA, sep=".")]))
 		assign(paste(tmp.state, "SGP_WIDE_Data", sep="_"), reshape(long_data_tmp["VALID_CASE"], idvar="ID", 
 			timevar="YEAR_BY_CONTENT_AREA", drop=c("VALID_CASE", "CONTENT_AREA", "YEAR"), direction="wide"))
+		eval(parse(text=paste("setkey(", tmp.state, "_SGP_WIDE_Data, ID)", sep="")))
 
 		save(list=paste(tmp.state, "SGP_WIDE_Data", sep="_"), file=file.path(outputSGP.directory, paste(tmp.state, "SGP_WIDE_Data.Rdata", sep="_")))
 		write.table(get(paste(tmp.state, "SGP_WIDE_Data", sep="_")), 
@@ -113,12 +124,19 @@ function(sgp_object,
 
 		if (identical(.Platform$OS.type, "unix")) {
 			if (file.info(file.path(outputSGP.directory, paste(tmp.state, "SGP_WIDE_Data.txt", sep="_")))$size > 4000000000) {
-				system(paste("gzip", file.path(outputSGP.directory, paste(tmp.state, "SGP_WIDE_Data.txt", sep="_"))))
+				tmp.working.directory <- getwd()
+				setwd(file.path(outputSGP.directory))
+				if (paste(tmp.state, "SGP_WIDE_Data.txt.gz", sep="_") %in% list.files()) file.remove(paste(tmp.state, "SGP_WIDE_Data.txt.gz", sep="_"))
+				system(paste("gzip", paste(tmp.state, "SGP_WIDE_Data.txt", sep="_")))
+				setwd(tmp.working.directory)
 			} else {
-				suppressWarnings(
-					zip(file.path(outputSGP.directory, paste(tmp.state, "SGP_WIDE_Data.txt.zip", sep="_")), 
-						file.path(outputSGP.directory, paste(tmp.state, "SGP_WIDE_Data.txt", sep="_")))
+				tmp.working.directory <- getwd()
+				setwd(file.path(outputSGP.directory))
+				if (paste(tmp.state, "SGP_WIDE_Data.txt.zip", sep="_") %in% list.files()) file.remove(paste(tmp.state, "SGP_WIDE_Data.txt.zip", sep="_"))
+				suppressMessages(
+					zip(paste(tmp.state, "SGP_WIDE_Data.txt.zip", sep="_"), paste(tmp.state, "SGP_WIDE_Data.txt", sep="_"))
 				)
+				setwd(tmp.working.directory)
 			}
 		}
 
@@ -153,7 +171,7 @@ function(sgp_object,
 		###
 
 			started.at <- proc.time()
-			message(paste("\tStarted SchoolVIEW WIDE data production in outputSGP", date()))
+			message(paste("\tStarted SchoolView STUDENT_GROWTH data production in outputSGP", date()))
 
 		### Utility functions
 
@@ -200,6 +218,41 @@ function(sgp_object,
 
 		"%w/o%" <- function(x,y) x[!x %in% y]
 
+		convert.variables <- function(tmp.df) {
+			if ("YEAR" %in% names(tmp.df) && is.character(tmp.df$YEAR)) {
+				tmp.df$YEAR <- as.integer(sapply(strsplit(tmp.df$YEAR, "_"), '[', 2))
+			}
+			if ("CONTENT_AREA" %in% names(tmp.df) && is.character(tmp.df$CONTENT_AREA)) {
+				tmp.df$CONTENT_AREA <- as.integer(as.factor(tmp.df$CONTENT_AREA))
+			}
+			if ("LAST_NAME" %in% names(tmp.df) && is.factor(tmp.df$LAST_NAME)) {
+				tmp.df$LAST_NAME <- as.character(tmp.df$LAST_NAME)
+			}
+			if ("FIRST_NAME" %in% names(tmp.df) && is.factor(tmp.df$FIRST_NAME)) {
+				tmp.df$FIRST_NAME <- as.character(tmp.df$FIRST_NAME)
+			}
+			if ("EMH_LEVEL" %in% names(tmp.df) && is.factor(tmp.df$EMH_LEVEL)) {
+				tmp.df[['EMH_LEVEL']] <- substr(tmp.df$EMH_LEVEL, 1, 1)
+			}
+			if ("GENDER" %in% names(tmp.df) && is.factor(tmp.df$GENDER)) {
+				tmp.df[['GENDER']] <- substr(tmp.df$GENDER, 1, 1)
+			}
+			for (names.iter in c(outputSGP.student.groups, "SCHOOL_ENROLLMENT_STATUS", "DISTRICT_ENROLLMENT_STATUS", "STATE_ENROLLMENT_STATUS") %w/o% "ETHNICITY") {
+				if (names.iter %in% names(tmp.df) && is.factor(tmp.df[[names.iter]])) {
+					tmp.df[[names.iter]] <- as.character(tmp.df[[names.iter]])
+					tmp.df[[names.iter]][grep("Yes", tmp.df[[names.iter]])] <- "Y"
+					tmp.df[[names.iter]][grep("No", tmp.df[[names.iter]])] <- "N"
+					tmp.df[[names.iter]][tmp.df[[names.iter]]=="Students with Disabilities (IEP)"] <- "Y"
+					tmp.df[[names.iter]][tmp.df[[names.iter]]=="Economically Disadvantaged"] <- "Y"
+					tmp.df[[names.iter]][tmp.df[[names.iter]]=="English Language Learners (ELL)"] <- "N"
+				}
+			}
+			return(tmp.df)
+		}
+
+		unclass.data.table <- function(my.data.table) {
+			as.data.table(lapply(convert.variables(subset(my.data.table, VALID_CASE=="VALID_CASE")), unclass))
+		}
 
 		#### Set key
 
@@ -212,10 +265,14 @@ function(sgp_object,
 		if (is.null(outputSGP_INDIVIDUAL.years)) {
 			tmp.years <- sort(unique(sgp_object@Data["VALID_CASE"][["YEAR"]]))
 			tmp.last.year <- tail(tmp.years, 1)
+			tmp.years.short <- sapply(strsplit(tmp.years, "_"), '[', 2)
+			tmp.last.year.short <- tail(unlist(strsplit(tail(tmp.years, 1), "_")), 1)
 		} else {
 			tmp.all.years <- sort(unique(sgp_object@Data["VALID_CASE"][["YEAR"]])) 
 			tmp.years <- tmp.all.years[1:which(tmp.all.years==tail(sort(outputSGP_INDIVIDUAL.years), 1))]
 			tmp.last.year <- tail(tmp.years, 1)
+			tmp.years.short <- sapply(strsplit(tmp.years, "_"), '[', 2)
+			tmp.last.year.short <- tail(unlist(strsplit(tail(tmp.years, 1), "_")), 1)
 		}
 
 
@@ -224,17 +281,23 @@ function(sgp_object,
 		if (is.null(outputSGP_INDIVIDUAL.content_areas)) {
 			tmp.content_areas <- sort(unique(sgp_object@Data[SJ("VALID_CASE", tmp.last.year)][["CONTENT_AREA"]]))
 		} else {
-			tmp.content_areas <- as.factor(outputSGP_INDIVIDUAL.content_areas)
+			tmp.content_areas <- sort(outputSGP_INDIVIDUAL.content_areas)
 		}
 
 
 		### subset data
 
 		tmp.districts.and.schools <- unique(data.table(sgp_object@Data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas)][,
-								list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER)], key=key(sgp_object)))
-		report.ids <- unique(sgp_object@Data[tmp.districts.and.schools][["ID"]])
-		setkeyv(sgp_object@Data, c("ID", "CONTENT_AREA", "YEAR"))
-		tmp.table <- sgp_object@Data[CJ(report.ids, tmp.content_areas, tmp.years)]
+								list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER, EMH_LEVEL)], key=key(sgp_object)))
+		report.ids <- data.table(sgp_object@Data[tmp.districts.and.schools][VALID_CASE=="VALID_CASE" & STATE_ENROLLMENT_STATUS=="Enrolled State: Yes"][, 
+			list(ID, FIRST_NAME, LAST_NAME, DISTRICT_NUMBER, SCHOOL_NUMBER, EMH_LEVEL)], key=c("ID", "FIRST_NAME", "LAST_NAME", "DISTRICT_NUMBER", "SCHOOL_NUMBER"))
+		setkey(report.ids, ID)
+		report.ids <- unique(report.ids)
+		setkeyv(sgp_object@Data, c("ID", "CONTENT_AREA", "YEAR", "VALID_CASE"))
+		tmp.table <- sgp_object@Data[CJ(report.ids[["ID"]], tmp.content_areas, tmp.years, "VALID_CASE")[report.ids]]
+		tmp.table[,FIRST_NAME:=NULL]; tmp.table[,LAST_NAME:=NULL]; tmp.table[,DISTRICT_NUMBER:=NULL]; tmp.table[,SCHOOL_NUMBER:=NULL]; tmp.table[,EMH_LEVEL:=NULL]
+		setnames(tmp.table, "FIRST_NAME.1", "FIRST_NAME"); setnames(tmp.table, "LAST_NAME.1", "LAST_NAME"); 
+		setnames(tmp.table, "DISTRICT_NUMBER.1", "DISTRICT_NUMBER"); setnames(tmp.table, "SCHOOL_NUMBER.1", "SCHOOL_NUMBER"); setnames(tmp.table, "EMH_LEVEL.1", "EMH_LEVEL")
 		setkeyv(sgp_object@Data, c("VALID_CASE", "CONTENT_AREA", "YEAR", "GRADE"))
 
 		### Create transformed scale scores
@@ -269,11 +332,10 @@ function(sgp_object,
 	### Reshape data set
 
 		variables.to.keep <- c("VALID_CASE", "ID", "LAST_NAME", "FIRST_NAME", "CONTENT_AREA", "YEAR", "GRADE", "EMH_LEVEL", 
-			"SCALE_SCORE", "TRANSFORMED_SCALE_SCORE", "ACHIEVEMENT_LEVEL", "SGP", "SGP_TARGET", "SCHOOL_NUMBER", "DISTRICT_NUMBER",
-			outputSGP.student.groups,
-			"SCHOOL_ENROLLMENT_STATUS", "DISTRICT_ENROLLMENT_STATUS", "STATE_ENROLLMENT_STATUS")
+			"SCALE_SCORE", "TRANSFORMED_SCALE_SCORE", "ACHIEVEMENT_LEVEL", "SGP", getTargetName(target.years=outputSGP.projection.years.for.target),
+			"SCHOOL_NUMBER", "DISTRICT_NUMBER", outputSGP.student.groups, "SCHOOL_ENROLLMENT_STATUS", "DISTRICT_ENROLLMENT_STATUS", "STATE_ENROLLMENT_STATUS")
 
-		outputSGP.data <- reshape(tmp.table[VALID_CASE=="VALID_CASE", variables.to.keep, with=FALSE],
+		outputSGP.data <- reshape(unclass.data.table(tmp.table)[, variables.to.keep, with=FALSE],
 			idvar=c("ID", "CONTENT_AREA"),
 			timevar="YEAR",
 			drop=c("VALID_CASE"),
@@ -291,15 +353,15 @@ function(sgp_object,
 					tmp.list[[i]] <- data.table(CONTENT_AREA=unlist(strsplit(i, "[.]"))[1],
 						sgp_object@SGP[["SGProjections"]][[i]][,c(1, grep(paste("PROJ_YEAR", j, sep="_"), names(sgp_object@SGP[["SGProjections"]][[i]])))])
 				}
-				outputSGP.data <- data.table(rbind.fill(tmp.list), key=paste(key(outputSGP.data), collapse=","))[outputSGP.data]
-				tmp.grade.name <- paste("GRADE", tmp.last.year, sep=".")
-				tmp.year.name <- .year.increment(tmp.last.year, j)
+				outputSGP.data <- data.table(convert.variables(rbind.fill(tmp.list)), key=paste(key(outputSGP.data), collapse=","))[outputSGP.data]
+				tmp.grade.name <- paste("GRADE", tmp.last.year.short, sep=".")
+				tmp.year.name <- .year.increment(tmp.last.year.short, j)
 				setkeyv(outputSGP.data, c("CONTENT_AREA", tmp.grade.name))
 				for (proj.iter in grep(paste("PROJ_YEAR", j, sep="_"), names(outputSGP.data))) {
-				tmp.scale_score.name <- names(outputSGP.data)[proj.iter]
-				outputSGP.data[[proj.iter]] <- outputSGP.data[,
-					piecewise.transform(get(tmp.scale_score.name), state, as.character(CONTENT_AREA[1]), tmp.year.name, get(tmp.grade.name)[1]+1), 
-					by=list(CONTENT_AREA, outputSGP.data[[tmp.grade.name]])]$V1 
+					tmp.scale_score.name <- names(outputSGP.data)[proj.iter]
+					outputSGP.data[[proj.iter]] <- outputSGP.data[,
+						piecewise.transform(get(tmp.scale_score.name), state, tmp.content_areas[CONTENT_AREA[1]], tmp.year.name, as.character(type.convert(get(tmp.grade.name)[1])+1)), 
+						by=list(CONTENT_AREA, outputSGP.data[[tmp.grade.name]])]$V1 
 				}
 			}
 		}
@@ -313,32 +375,49 @@ function(sgp_object,
 		## Rename variables to keep
 
 		setnames(outputSGP.data, which(names(outputSGP.data)=="ID"), "STATE_ASSIGNED_ID")
-		setnames(outputSGP.data, which(names(outputSGP.data)==paste("LAST_NAME", tmp.last.year, sep=".")), "LAST_NAME")
-		setnames(outputSGP.data, which(names(outputSGP.data)==paste("FIRST_NAME", tmp.last.year, sep=".")), "FIRST_NAME")
-		setnames(outputSGP.data, which(names(outputSGP.data)==paste("DISTRICT_NUMBER", tmp.last.year, sep=".")), "DISTRICT_NUMBER")
-		setnames(outputSGP.data, which(names(outputSGP.data)==paste("SCHOOL_NUMBER", tmp.last.year, sep=".")), "SCHOOL_NUMBER")
-		setnames(outputSGP.data, which(names(outputSGP.data)==paste("EMH_LEVEL", tmp.last.year, sep=".")), "EMH_LEVEL")
+		setnames(outputSGP.data, which(names(outputSGP.data)==paste("LAST_NAME", tmp.last.year.short, sep=".")), "LAST_NAME")
+		setnames(outputSGP.data, which(names(outputSGP.data)==paste("FIRST_NAME", tmp.last.year.short, sep=".")), "FIRST_NAME")
+		setnames(outputSGP.data, which(names(outputSGP.data)==paste("DISTRICT_NUMBER", tmp.last.year.short, sep=".")), "DISTRICT_NUMBER")
+		setnames(outputSGP.data, which(names(outputSGP.data)==paste("SCHOOL_NUMBER", tmp.last.year.short, sep=".")), "SCHOOL_NUMBER")
+		setnames(outputSGP.data, which(names(outputSGP.data)==paste("EMH_LEVEL", tmp.last.year.short, sep=".")), "EMH_LEVEL")
 		for (i in outputSGP.student.groups) {
-			setnames(outputSGP.data, which(names(outputSGP.data)==paste(i, tmp.last.year, sep=".")), i)
+			setnames(outputSGP.data, which(names(outputSGP.data)==paste(i, tmp.last.year.short, sep=".")), i)
 		}
-		setnames(outputSGP.data, which(names(outputSGP.data)==paste("SCHOOL_ENROLLMENT_STATUS", tmp.last.year, sep=".")), "SCHOOL_ENROLLMENT_STATUS")
-		setnames(outputSGP.data, which(names(outputSGP.data)==paste("DISTRICT_ENROLLMENT_STATUS", tmp.last.year, sep=".")), "DISTRICT_ENROLLMENT_STATUS")
-		setnames(outputSGP.data, which(names(outputSGP.data)==paste("STATE_ENROLLMENT_STATUS", tmp.last.year, sep=".")), "STATE_ENROLLMENT_STATUS")
+		setnames(outputSGP.data, which(names(outputSGP.data)==paste("SCHOOL_ENROLLMENT_STATUS", tmp.last.year.short, sep=".")), "SCHOOL_ENROLLMENT_STATUS")
+		setnames(outputSGP.data, which(names(outputSGP.data)==paste("DISTRICT_ENROLLMENT_STATUS", tmp.last.year.short, sep=".")), "DISTRICT_ENROLLMENT_STATUS")
+		setnames(outputSGP.data, which(names(outputSGP.data)==paste("STATE_ENROLLMENT_STATUS", tmp.last.year.short, sep=".")), "STATE_ENROLLMENT_STATUS")
 
-		for (i in seq_along(tmp.years)) {	
-			setnames(outputSGP.data, grep(paste("GRADE", rev(tmp.years)[i], sep="."), names(outputSGP.data)), paste("GRADE_LEVEL", tmp.order[i], sep="_"))
-			setnames(outputSGP.data, grep(paste("SCALE_SCORE", rev(tmp.years)[i], sep="."), names(outputSGP.data)) %w/o% 
-				grep(paste("TRANSFORMED_SCALE_SCORE", rev(tmp.years)[i], sep="."), names(outputSGP.data)), 
+		if ("ELL_STATUS" %in% outputSGP.student.groups) {
+			setnames(outputSGP.data, which(names(outputSGP.data)=="ELL_STATUS"), "LANGUAGE_PROFICIENCY")
+		} else {
+			outputSGP.data[['LANGUAGE_PROFICIENCY']] <- as.character(NA)
+		}
+		if ("GIFTED_AND_TALENTED_PROGRAM_STATUS" %in% outputSGP.student.groups) {
+			setnames(outputSGP.data, which(names(outputSGP.data)=="GIFTED_AND_TALENTED_PROGRAM_STATUS"), "GIFTED_CODE")
+		} else {
+			outputSGP.data[['GIFTED_CODE']] <- as.character(NA)
+		}
+		if ("HOMELESS_STATUS" %in% outputSGP.student.groups) {
+			setnames(outputSGP.data, which(names(outputSGP.data)=="HOMELESS_STATUS"), "HLS_CODE")
+		} else {
+			outputSGP.data[['HLS_CODE']] <- as.character(NA)
+		}
+
+		for (i in seq_along(tmp.years.short)) {	
+			setnames(outputSGP.data, grep(paste("GRADE", rev(tmp.years.short)[i], sep="."), names(outputSGP.data)), paste("GRADE_LEVEL", tmp.order[i], sep="_"))
+			setnames(outputSGP.data, grep(paste("SCALE_SCORE", rev(tmp.years.short)[i], sep="."), names(outputSGP.data)) %w/o% 
+				grep(paste("TRANSFORMED_SCALE_SCORE", rev(tmp.years.short)[i], sep="."), names(outputSGP.data)), 
 				paste("SCALE_SCORE", tmp.order[i], sep="_"))
-			setnames(outputSGP.data, grep(paste("TRANSFORMED_SCALE_SCORE", rev(tmp.years)[i], sep="."), names(outputSGP.data)), paste("TRANSFORMED_SCALE_SCORE", tmp.order[i], sep="_"))
-			setnames(outputSGP.data, grep(paste("SGP_TARGET", rev(tmp.years)[i], sep="."), names(outputSGP.data)), paste("GROWTH_TARGET", tmp.order[i], sep="_"))
-			setnames(outputSGP.data, grep(paste("SGP", rev(tmp.years)[i], sep="."), names(outputSGP.data)), paste("GROWTH_PERCENTILE", tmp.order[i], sep="_"))
-			setnames(outputSGP.data, grep(paste("ACHIEVEMENT_LEVEL", rev(tmp.years)[i], sep="."), names(outputSGP.data)), paste("PERFORMANCE_LEVEL", tmp.order[i], sep="_"))
+			setnames(outputSGP.data, grep(paste("TRANSFORMED_SCALE_SCORE", rev(tmp.years.short)[i], sep="."), names(outputSGP.data)), paste("TRANSFORMED_SCALE_SCORE", tmp.order[i], sep="_"))
+			setnames(outputSGP.data, grep(paste("SGP", rev(tmp.years.short)[i], sep="."), names(outputSGP.data)), paste("GROWTH_PERCENTILE", tmp.order[i], sep="_"))
+			setnames(outputSGP.data, grep(paste("ACHIEVEMENT_LEVEL", rev(tmp.years.short)[i], sep="."), names(outputSGP.data)), paste("PERFORMANCE_LEVEL", tmp.order[i], sep="_"))
+			setnames(outputSGP.data, grep(paste(getTargetName(target.years=outputSGP.projection.years.for.target), rev(tmp.years.short)[i], sep="."), names(outputSGP.data)), 
+				paste("GROWTH_TARGET", tmp.order[i], sep="_"))
 		}
 	
 		## NULLify variable to be removed
 
-		for (i in head(tmp.years, -1)) {
+		for (i in head(tmp.years.short, -1)) {
 			outputSGP.data[[paste("LAST_NAME", i, sep=".")]] <- NULL
 			outputSGP.data[[paste("FIRST_NAME", i, sep=".")]] <- NULL
 			outputSGP.data[[paste("DISTRICT_NUMBER", i, sep=".")]] <- NULL
@@ -350,19 +429,39 @@ function(sgp_object,
 			outputSGP.data[[paste("SCHOOL_ENROLLMENT_STATUS", i, sep=".")]] <- NULL
 			outputSGP.data[[paste("DISTRICT_ENROLLMENT_STATUS", i, sep=".")]] <- NULL
 			outputSGP.data[[paste("STATE_ENROLLMENT_STATUS", i, sep=".")]] <- NULL
+	
+			if ("ELL_STATUS" %in% outputSGP.student.groups) {
+				outputSGP.data[[paste("ELL_STATUS", i, sep=".")]] <- NULL
+				outputSGP.student.groups[outputSGP.student.groups=="ELL_STATUS"] <- "LANGUAGE_PROFICIENCY"
+			} else {
+				outputSGP.student.groups <- c(outputSGP.student.groups, "LANGUAGE_PROFICIENCY")
+			}
+	
+			if ("GIFTED_AND_TALENTED_PROGRAM_STATUS" %in% outputSGP.student.groups) {
+				outputSGP.data[[paste("GIFTED_AND_TALENTED_PROGRAM_STATUS", i, sep=".")]] <- NULL
+				outputSGP.student.groups[outputSGP.student.groups=="GIFTED_AND_TALENTED_PROGRAM_STATUS"] <- "GIFTED_CODE"
+			} else {
+				outputSGP.student.groups <- c(outputSGP.student.groups, "GIFTED_CODE")
+			}
+	
+			if ("HOMELESS_STATUS" %in% outputSGP.student.groups) {
+				outputSGP.data[[paste("HOMELESS_STATUS", i, sep=".")]] <- NULL
+				outputSGP.student.groups[outputSGP.student.groups=="HOMELESS_STATUS"] <- "HLS_CODE"
+			} else {
+				outputSGP.student.groups <- c(outputSGP.student.groups, "HLS_CODE")
+			}
 		}
 
 
 		## Create missing variables
 
-		outputSGP.data[["YEAR"]] <- tmp.last.year
+		outputSGP.data[["YEAR"]] <- tmp.last.year.short
 		outputSGP.data[["STUDENT_GROWTH_ID"]] <- seq(dim(outputSGP.data)[1])
 		outputSGP.data[["MIDDLE_NAME"]] <- as.character(NA)
-		outputSGP.data[["HLS_CODE"]] <- as.character(NA)
 		outputSGP.data[["OCTOBER_ENROLLMENT_STATUS"]] <- as.character(NA)
 
-		if (length(tmp.years) < length(tmp.order)) {
-			for (i in tmp.order[(length(tmp.years)+1):length(tmp.order)]) {
+		if (length(tmp.years.short) < length(tmp.order)) {
+			for (i in tmp.order[(length(tmp.years.short)+1):length(tmp.order)]) {
 				outputSGP.data[[paste("GRADE_LEVEL", i, sep="_")]] <- NA
 				outputSGP.data[[paste("SCALE_SCORE", i, sep="_")]] <- NA
 				outputSGP.data[[paste("TRANSFORMED_SCALE_SCORE", i, sep="_")]] <- NA
@@ -391,11 +490,12 @@ function(sgp_object,
 
 		## Rearrange variables
 
+		tmp.gt.name <- getTargetName(target.years=outputSGP.projection.years.for.target, target.label="GROWTH_TARGET")
 		tmp.variable.names <- c("STUDENT_GROWTH_ID", "STATE_ASSIGNED_ID", "LAST_NAME", "FIRST_NAME", "MIDDLE_NAME", 
 			"CONTENT_AREA", "YEAR", "DISTRICT_NUMBER", "SCHOOL_NUMBER", "EMH_LEVEL",
-			outputSGP.student.groups,
+			unique(outputSGP.student.groups),
 			"OCTOBER_ENROLLMENT_STATUS", "SCHOOL_ENROLLMENT_STATUS", "DISTRICT_ENROLLMENT_STATUS", "STATE_ENROLLMENT_STATUS",
-			"GRADE_LEVEL_CY", "SCALE_SCORE_CY", "TRANSFORMED_SCALE_SCORE_CY", "PERFORMANCE_LEVEL_CY", "GROWTH_PERCENTILE_CY", "GROWTH_TARGET_CY",
+			"GRADE_LEVEL_CY", "SCALE_SCORE_CY", "TRANSFORMED_SCALE_SCORE_CY", "PERFORMANCE_LEVEL_CY", "GROWTH_PERCENTILE_CY", "GROWTH_TARGET_CY", 
 			"GRADE_LEVEL_PY1", "SCALE_SCORE_PY1", "TRANSFORMED_SCALE_SCORE_PY1", "PERFORMANCE_LEVEL_PY1", "GROWTH_PERCENTILE_PY1", "GROWTH_TARGET_PY1",
 			"GRADE_LEVEL_PY2", "SCALE_SCORE_PY2", "TRANSFORMED_SCALE_SCORE_PY2", "PERFORMANCE_LEVEL_PY2", "GROWTH_PERCENTILE_PY2", "GROWTH_TARGET_PY2",
 			"GRADE_LEVEL_PY3", "SCALE_SCORE_PY3", "TRANSFORMED_SCALE_SCORE_PY3", "PERFORMANCE_LEVEL_PY3", "GROWTH_PERCENTILE_PY3", "GROWTH_TARGET_PY3",
@@ -407,16 +507,19 @@ function(sgp_object,
 			"CUT_1_YEAR_2", "CUT_99_YEAR_2", "CUT_35_YEAR_2", "CUT_65_YEAR_2", "CUT_20_YEAR_2", "CUT_40_YEAR_2", "CUT_60_YEAR_2", "CUT_80_YEAR_2",
 			"CUT_1_YEAR_3", "CUT_99_YEAR_3", "CUT_35_YEAR_3", "CUT_65_YEAR_3", "CUT_20_YEAR_3", "CUT_40_YEAR_3", "CUT_60_YEAR_3", "CUT_80_YEAR_3")
 
-		write.table(outputSGP.data[,tmp.variable.names, with=FALSE], file=file.path(outputSGP.directory, "SchoolView", "SchoolView_WIDE.txt"), row.names=FALSE, na="", quote=FALSE, sep="|")
-			suppressWarnings(
-				zip(file.path(outputSGP.directory, "SchoolView", "SchoolView_WIDE.txt.zip"), 
-					file.path(outputSGP.directory, "SchoolView", "SchoolView_WIDE.txt"))
+		write.table(outputSGP.data[,tmp.variable.names, with=FALSE], file=file.path(outputSGP.directory, "SchoolView", "STUDENT_GROWTH.dat"), row.names=FALSE, na="", quote=FALSE, sep="|")
+			tmp.working.directory <- getwd()
+			setwd(file.path(outputSGP.directory, "SchoolView"))
+			if ("STUDENT_GROWTH.dat.zip" %in% list.files()) file.remove("STUDENT_GROWTH.dat.zip")
+			suppressMessages(
+				zip("STUDENT_GROWTH.dat.zip", "STUDENT_GROWTH.dat")
 			)
+			setwd(tmp.working.directory)
 
-		SchoolView_WIDE <- outputSGP.data[,tmp.variable.names, with=FALSE]
-		save(SchoolView_WIDE, file=file.path(outputSGP.directory, "SchoolView", "SchoolView_WIDE.Rdata"))
+		STUDENT_GROWTH <- outputSGP.data[,tmp.variable.names, with=FALSE]
+		save(STUDENT_GROWTH, file=file.path(outputSGP.directory, "SchoolView", "STUDENT_GROWTH.Rdata"))
 
-		message(paste("\tFinished SchoolVIEW WIDE data production in outputSGP", date(), "in", timetaken(started.at), "\n"))
+		message(paste("\tFinished SchoolView STUDENT_GROWTH data production in outputSGP", date(), "in", timetaken(started.at), "\n"))
 
 	} ## End if SchoolView %in% output.type
 
