@@ -1,16 +1,18 @@
 `studentGrowthProjections` <-
 function(panel.data,	## REQUIRED
 	sgp.labels,	## REQUIRED  
-	grade.progression,	## REQUIRED  
+	grade.progression,	## REQUIRED
+	content_area.progression=NULL,
+	grade.projection.sequence=NULL,
+	content_area.projection.sequence=NULL,
 	max.forward.progression.years,
 	max.forward.progression.grade,
-	max.order.for.progression,
+	max.order.for.progression=NULL,
 	use.my.knots.boundaries,
 	use.my.coefficient.matrices,
 	panel.data.vnames,
 	achievement.level.prior.vname=NULL,
 	performance.level.cutscores,
-	chunk.size=100000,
         calculate.sgps=TRUE,
 	convert.0and100=TRUE,
 	projection.unit="YEAR",
@@ -47,7 +49,7 @@ function(panel.data,	## REQUIRED
                 sub(' ', '_', toupper(sub('\\.+$', '', paste(unlist(sapply(labels[pieces], as.character)), collapse="."))))
         }
 
-        get.my.knots.boundaries.path <- function(content_area, year) {
+	get.my.knots.boundaries.path <- function(content_area, year) {
 		tmp.knots.boundaries.names <- names(panel.data[["Knots_Boundaries"]][[tmp.path.knots.boundaries]])[
 			grep(content_area, names(panel.data[["Knots_Boundaries"]][[tmp.path.knots.boundaries]]))]
 		if (length(tmp.knots.boundaries.names)==0) {
@@ -63,26 +65,13 @@ function(panel.data,	## REQUIRED
 					} else {
 						return(paste("[['", tmp.path.knots.boundaries, "']][['", content_area, ".", rev(sort(tmp.knots.boundaries.years))[1], "']]", sep=""))
 					}
-                                }
-                        } else {
-                                return(paste("[['", tmp.path.knots.boundaries, "']][['", content_area, "']]", sep=""))
-                        }
+				}
+			} else {
+				return(paste("[['", tmp.path.knots.boundaries, "']][['", content_area, "']]", sep=""))
+			}
 		}
-        }
-
-	.get.max.matrix.order <- function(names, grade) {
-		tmp <- do.call(rbind, strsplit(names, "_"))
-		tmp.vec <- vector("numeric", length(grade))
-		for (i in seq_along(grade)) {
-			tmp.vec[i] <- max(as.numeric(tmp[tmp[,2]==grade[i],3]), na.rm=TRUE)
-		}
-		return(tmp.vec)
 	}
 
-	.get.max.matrix.grade <- function(names, grade) {
-		tmp <- do.call(rbind, strsplit(names, "_"))
-		max(as.numeric(tmp[,2]), na.rm=TRUE)
-	}
 
 	.get.panel.data <- function(data, num.prior.scores, by.grade, subset.tf, tmp.gp) {
 		if (missing(subset.tf)) {
@@ -113,27 +102,27 @@ function(panel.data,	## REQUIRED
 		}
 	}
 
-	.get.data.table <- function(ss.data) {
-		names(ss.data) <- NA
-		names(ss.data)[c(1, (1+num.panels-max(num.predictors)+1):(1+num.panels), (1+2*num.panels-max(num.predictors)+1):(1+2*num.panels))] <- 
-			c("ORIGINAL.ID", GD, SS)
-		data.table(ID=seq(dim(ss.data)[1]), ss.data, key="ID")
-	}
+#	.get.data.table <- function(ss.data) {
+#		names(ss.data) <- NA
+#		names(ss.data)[c(1, (1+num.panels-max(num.predictors)+1):(1+num.panels), (1+2*num.panels-max(num.predictors)+1):(1+2*num.panels))] <- 
+#			c("ORIGINAL.ID", GD, SS)
+#		data.table(ID=seq(dim(ss.data)[1]), ss.data, key="ID")
+#	}
 
-	.unget.data.table <- function(my.data, my.lookup) {
-		setkey(my.data, ID); ORIGINAL.ID <- NULL
-		my.data[["ID"]] <- my.lookup[my.data[["ID"]], ORIGINAL.ID]
-		if (!is.null(achievement.level.prior.vname)) {
-			panel.data[["Panel_Data"]] <- as.data.table(panel.data[["Panel_Data"]])
-			setkey(panel.data[["Panel_Data"]], ID)
-			invisible(setkeyv(my.data, NULL)); setkey(my.data, ID)
-			my.data <- panel.data[["Panel_Data"]][,c("ID", achievement.level.prior.vname), with=FALSE][my.data]
-			setnames(my.data, 2, "ACHIEVEMENT_LEVEL_PRIOR")
-		}
-		return(as.data.frame(my.data))
-	}
+#	.unget.data.table <- function(my.data, my.lookup) {
+#		setkey(my.data, ID); ORIGINAL.ID <- NULL
+#		my.data[["ID"]] <- my.lookup[my.data[["ID"]], ORIGINAL.ID]
+#		if (!is.null(achievement.level.prior.vname)) {
+#			panel.data[["Panel_Data"]] <- as.data.table(panel.data[["Panel_Data"]])
+#			setkey(panel.data[["Panel_Data"]], ID)
+#			invisible(setkeyv(my.data, NULL)); setkey(my.data, ID)
+#			my.data <- panel.data[["Panel_Data"]][,c("ID", achievement.level.prior.vname), with=FALSE][my.data]
+#			setnames(my.data, 2, "ACHIEVEMENT_LEVEL_PRIOR")
+#		}
+#		return(as.data.frame(my.data))
+#	}
 
-	.year.increment <- function(year, increment, lag) {
+	.year.increment <- function(year, increment, lag=0) {
 		paste(as.numeric(unlist(strsplit(as.character(year), "_")))+increment-lag, collapse="_")	
 	}
 
@@ -160,79 +149,115 @@ function(panel.data,	## REQUIRED
 		if (!order %in% as.numeric(tmp[tmp[,2]==grade,3])) stop(paste("Coefficient matrix associated with grade ", grade, "order ", order, " not found.", sep=""))
 	}
 
-	.get.grade.projection.sequence.priors <- function(grade.progression, grade.projection.sequence, max.order.tf) {
-		tmp.list <- vector("list", length(grade.progression))
-		for (i in 1:length(grade.progression)) {
-			tmp.list[[i]] <- .get.max.matrix.order(matrix.names, grade.projection.sequence)
+	get.grade.projection.sequence.priors <- function(grade.progression, grade.projection.sequence) {
+
+		.get.max.matrix.order <- function(names, grade) {
+			tmp <- do.call(rbind, strsplit(names, "_"))
+			tmp.vec <- vector("numeric", length(grade))
+			for (i in seq_along(grade)) {
+				tmp.vec[i] <- max(as.numeric(tmp[tmp[,2]==grade[i],3]), na.rm=TRUE)
+			}
+			return(tmp.vec)
+		}
+
+		tmp.list <- list()
+		for (i in seq_along(grade.progression)) {
+
+			tmp.list[[i]] <- .get.max.matrix.order(names(tmp.matrices), grade.projection.sequence)
 			tmp.list[[i]] <- pmin(tmp.list[[i]], seq(i, length.out=length(grade.projection.sequence)))
-			if (!max.order.tf) {
+			if (!is.null(max.order.for.progression)) {
 				tmp.list[[i]] <- pmin(tmp.list[[i]], rep(max.order.for.progression, length(grade.projection.sequence)))
 			}
 		}
 		return(rev(tmp.list[!duplicated(tmp.list)]))
 	}
 
+	.get.coefficient.matrix <- function(grade, order, content.areas, grade.prog) {
+		tmp.mtx.name <- paste("qrmatrix", grade, order, sep="_") 
+		tmp.index <- grep(tmp.mtx.name, names(tmp.matrices))
+		tmp.tf <- tmp.index2 <- NULL
+		for (i in tmp.index) {
+			if (!identical(class(try(panel.data[['Coefficient_Matrices']][[tmp.path.coefficient.matrices]][[i]]@Content_Areas, silent=TRUE)), "try-error")) {
+				tmp.tf <- c(tmp.tf, TRUE); tmp.index2 <- c(tmp.index2, i)
+			} else tmp.tf <- c(tmp.tf, FALSE)
+		}
+		if (any(tmp.tf)) {
+			for (i in tmp.index2) {
+				if (all(panel.data[['Coefficient_Matrices']][[tmp.path.coefficient.matrices]][[i]]@Content_Areas[[1]] == content.areas) & 
+				    all(panel.data[['Coefficient_Matrices']][[tmp.path.coefficient.matrices]][[i]]@Grade_Progression[[1]] == grade.prog)) {
+					tmp.mtx <- panel.data[['Coefficient_Matrices']][[tmp.path.coefficient.matrices]][[i]]	
+				}
+			}
+		} else {
+			tmp.mtx <- panel.data[['Coefficient_Matrices']][[tmp.path.coefficient.matrices]][[tmp.mtx.name]]
+		}
+
+		if (dim(tmp.mtx)[2] != 100) {
+			tau.num <- ceiling(as.numeric(substr(colnames(tmp.mtx), 6, nchar(colnames(tmp.mtx))))*100)
+			na.replace <- 1:100 %in% tau.num
+			na.mtx <- matrix(NA, nrow=nrow(tmp.mtx), ncol=100)
+			na.mtx[,na.replace] <- tmp.mtx
+			tmp.mtx <- na.mtx
+			missing.taus=TRUE
+		}
+		return(tmp.mtx)
+	}
+
 	.get.percentile.trajectories <- function(ss.data) {
 
-		tmp.percentile.trajectories <- vector("list", length(grade.projection.sequence.priors))
-		completed.ids <- NULL
+		tmp.percentile.trajectories <- list()
+		completed.ids <- TEMP_1 <- TEMP_2 <-NULL
 
 		for (i in seq_along(grade.projection.sequence.priors)) {
 			tmp.gp <- tail(grade.progression, grade.projection.sequence.priors[[i]][1])
 			if (any(!ss.data[["ID"]] %in% completed.ids)) {
-				tmp.dim <- dim(.get.panel.data(ss.data, grade.projection.sequence.priors[[i]][1], by.grade, subset.tf=!(ss.data[["ID"]] %in% completed.ids), tmp.gp=tmp.gp))
-				tmp.storage.matrix <- matrix(nrow=100*tmp.dim[1], ncol=length(grade.projection.sequence)+tmp.dim[2])
-				tmp.storage.matrix[,seq(tmp.dim[2])] <- as.matrix(sapply(.get.panel.data(ss.data, 
-					grade.projection.sequence.priors[[i]][1], by.grade, subset.tf=!(ss.data[["ID"]] %in% completed.ids), tmp.gp=tmp.gp), rep, each=100))
-				colnames(tmp.storage.matrix) <- c("ID", paste("SS", c(tmp.gp, grade.projection.sequence), sep=""))
-				completed.ids <- c(unique(tmp.storage.matrix[,"ID"]), completed.ids)
-				missing.taus=FALSE; na.replace=NULL # put these outside of j loop so that stay's true/non-null if only SOME of coef matrices have missing column/taus.
+				tmp.dt <- .get.panel.data(ss.data, grade.projection.sequence.priors[[i]][1], by.grade, subset.tf=!(ss.data[["ID"]] %in% completed.ids), tmp.gp=tmp.gp)
+				if (dim(tmp.dt)[1] > 0) {
+					setkey(tmp.dt, ID)
+					completed.ids <- c(unique(tmp.dt[["ID"]]), completed.ids)
+					tmp.dt <- tmp.dt[list(rep(ID, each=100))]
+					missing.taus <- FALSE; na.replace <- NULL # put these outside of j loop so that stays true/non-null if only SOME of coef matrices have missing column/taus.
 
-				for (j in seq_along(grade.projection.sequence.priors[[i]])) {
-					mod <- character()
-					int <- "cbind(rep(1, 100*tmp.dim[1]),"
-					for (k in 1:grade.projection.sequence.priors[[i]][j]) {
-						knt <- paste("panel.data[['Coefficient_Matrices']][['", tmp.path.coefficient.matrices, "']][['qrmatrix_", grade.projection.sequence[j], "_", grade.projection.sequence.priors[[i]][j], "']]@Knots[['knots_", rev(tmp.gp)[k], "']]", sep="")
-						bnd <- paste("panel.data[['Coefficient_Matrices']][['", tmp.path.coefficient.matrices, "']][['qrmatrix_", grade.projection.sequence[j], "_", grade.projection.sequence.priors[[i]][j], "']]@Boundaries[['boundaries_", rev(tmp.gp)[k], "']]", sep="")
-						mod <- paste(mod, ", bs(tmp.storage.matrix[,'SS", rev(tmp.gp)[k], "'], knots=", knt, ", Boundary.knots=", bnd, ")", sep="")
-					}
-					mat <- paste("panel.data[['Coefficient_Matrices']][['", tmp.path.coefficient.matrices, "']][['qrmatrix_", grade.projection.sequence[j], "_", k, "']]", sep="")
-					.check.my.coefficient.matrices(matrix.names, grade.projection.sequence[j], k)
-					tmp.scores <- eval(parse(text=paste(int, substring(mod, 2), ")", sep="")))
-					tmp.matrix <- eval(parse(text=mat))
-					if (dim(tmp.matrix)[2] != 100) {
-						tau.num <- ceiling(as.numeric(substr(colnames(tmp.matrix), 6, nchar(colnames(tmp.matrix))))*100)
-						na.replace <- 1:100 %in% tau.num
-						na.mtx <- matrix(NA, nrow=nrow(tmp.matrix), ncol=100)
-						na.mtx[,na.replace] <- tmp.matrix
-						tmp.matrix <- na.mtx
-						missing.taus=TRUE
-					}
-					if (identical(floor(tmp.dim[1]/chunk.size), tmp.dim[1]/chunk.size)) {
-						num.chunks <- floor(tmp.dim[1]/chunk.size) - 1
-					} else {
-						num.chunks <- floor(tmp.dim[1]/chunk.size)
-					} 
-					chunk.list <- vector("list", num.chunks+1)
-					for (chunk in 0:num.chunks) {
-						lower.index <- chunk*chunk.size
-						upper.index <- min((chunk+1)*chunk.size, tmp.dim[1])
-						quantile.list <- vector("list", 100)
-						for (m in 1:100) {
-							quantile.list[[m]] <-  tmp.scores[m+lower.index:(upper.index-1)*100,] %*% tmp.matrix[,m] 
+					for (j in seq_along(grade.projection.sequence.priors[[i]])) {
+						mod <- character()
+						int <- "cbind(rep(1, dim(tmp.dt)[1]),"
+						for (k in 1:grade.projection.sequence.priors[[i]][j]) {
+							knt <- paste("tmp.matrix@Knots[['knots_", rev(tmp.gp)[k], "']]", sep="")
+							bnd <- paste("tmp.matrix@Boundaries[['boundaries_", rev(tmp.gp)[k], "']]", sep="")
+							mod <- paste(mod, ", bs(tmp.dt[['SS", rev(tmp.gp)[k], "']], knots=", knt, ", Boundary.knots=", bnd, ")", sep="")
 						}
-						chunk.list[[chunk+1]] <- apply(matrix(unlist(quantile.list), ncol=100), 1, 
-							function(x) .smooth.bound.iso.row(x, grade.projection.sequence[j], .year.increment(sgp.labels$my.year, j, lag.increment),
-								missing.taus=missing.taus, na.replace=na.replace))
-					}
-					tmp.storage.matrix[,tmp.dim[2]+j] <- as.vector(do.call(cbind, chunk.list))
-					tmp.gp <- c(tmp.gp, grade.projection.sequence[j])
-					rm(list=c("tmp.scores", "chunk.list", "quantile.list")); suppressMessages(gc())
-				} ## END j loop
-				tmp.percentile.trajectories[[i]] <- tmp.storage.matrix[,-(1:grade.projection.sequence.priors[[i]][1]+1)]
+						.check.my.coefficient.matrices(names(tmp.matrices), grade.projection.sequence[j], k)
+						tmp.grd <- grade.projection.sequence[j]
+						tmp.matrix <-  .get.coefficient.matrix(
+									tmp.grd, 
+									order=grade.projection.sequence.priors[[i]][j], 
+									content.areas=sgp.labels$my.subject, 
+									grade.prog = tail(c(tmp.gp,tmp.grd), k+1))
+
+						tmp.scores <- eval(parse(text=paste(int, substring(mod, 2), ")", sep="")))
+
+						for (m in seq(100)) {
+							tmp.dt[m+100*(seq(dim(tmp.dt)[1]/100)-1), TEMP_1:=tmp.scores[m+100*(seq(dim(tmp.dt)[1]/100)-1),] %*% tmp.matrix[,m]]
+						}
+
+						tmp.dt[,TEMP_2:=tmp.dt[,.smooth.bound.iso.row(
+										TEMP_1, 
+										grade.projection.sequence[j], 
+										.year.increment(sgp.labels[['my.year']], j, lag.increment),  
+										missing.taus=missing.taus, 
+										na.replace=na.replace), 
+									by=ID][['V1']]]
+						setnames(tmp.dt, "TEMP_2", paste("SS", grade.projection.sequence[j], sep=""))
+						tmp.dt[,TEMP_1:=NULL]
+
+						tmp.gp <- c(tmp.gp, grade.projection.sequence[j])
+					} ## END j loop
+					tmp.percentile.trajectories[[i]] <- tmp.dt[,-(1:grade.projection.sequence.priors[[i]][1]+1),with=FALSE]
+					rm(tmp.dt); suppressMessages(gc())
+				} ## END if (dim(tmp.dt)[1] > 0)
 			} ## END if statement
 		} ## END i loop
-		as.data.frame(do.call(rbind, tmp.percentile.trajectories))
+		rbindlist(tmp.percentile.trajectories)
 	} ## END function
 
 	.sgp.targets <- function(data, cut, convert.0and100) {
@@ -248,8 +273,8 @@ function(panel.data,	## REQUIRED
 
 	.get.trajectories.and.cuts <- function(percentile.trajectories, trajectories.tf, cuts.tf, projection.unit=projection.unit) {
 		if (trajectories.tf) {
-			tmp.traj <- round(percentile.trajectories[seq(dim(percentile.trajectories)[1]) %% 100 %in% ((percentile.trajectory.values+1) %% 100), 
-					colnames(percentile.trajectories)], digits=projcuts.digits)
+			tmp.traj <- percentile.trajectories[seq(dim(percentile.trajectories)[1]) %% 100 %in% ((percentile.trajectory.values+1) %% 100)]
+			tmp.traj[,2:dim(tmp.traj)[2] := round(tmp.traj[,2:dim(tmp.traj)[2], with=FALSE], digits=projcuts.digits), with=FALSE]
 			trajectories <- data.table(reshape(data.table(tmp.traj, CUT=rep(percentile.trajectory.values, dim(tmp.traj)[1]/length(percentile.trajectory.values))), 
 				idvar="ID", timevar="CUT", direction="wide"), key="ID")
 			if (projection.unit=="GRADE") {
@@ -262,9 +287,7 @@ function(panel.data,	## REQUIRED
 			if (!cuts.tf) return(trajectories)
 		}
 		if (cuts.tf) {
-			percentile.trajectories[["ID"]] <- as.integer(percentile.trajectories[["ID"]]) 
-			percentile.trajectories <- data.table(percentile.trajectories, key="ID")
-
+			setkey(percentile.trajectories, ID)
 			k <- 1
 			cuts.arg <- names.arg <- character()
 
@@ -341,6 +364,15 @@ function(panel.data,	## REQUIRED
 		stop("User must supply a grade progression from which projections/trajectories will be derived. See help page for details.")
 	}
 
+	if (!is.numeric(type.convert(as.character(grade.progression)))) {
+		stop("The current implementation of studentGrowthProjections does not allcw for non-numeric grades. Stay tuned for a better implementation.")
+	}
+	grade.progression <- type.convert(as.character(grade.progression))
+
+	if (missing(content_area.progression)) {
+		content_area.progression <- rep(sgp.labels[['my.subject']], length(grade.progression))
+	}
+
 	if (!missing(use.my.knots.boundaries)) {
 		if (!is.list(use.my.knots.boundaries) & !is.character(use.my.knots.boundaries)) {
 			stop("use.my.knots.boundaries must be supplied as a list or character abbreviation. See help page for details.")
@@ -358,7 +390,7 @@ function(panel.data,	## REQUIRED
 			}
 		}
 		if (is.character(use.my.knots.boundaries)) {
-			if (!use.my.knots.boundaries %in% names(SGPstateData)) {
+			if (!use.my.knots.boundaries %in% ls(SGPstateData)) {
 				stop(paste("Knots and Boundaries are currently not implemented for the state (", use.my.knots.boundaries, ") indicated. Please contact the SGP package administrator to have your Knots and Boundaries included in the package", sep=""))
 			}
 		}
@@ -386,7 +418,7 @@ function(panel.data,	## REQUIRED
 
 	if (!missing(performance.level.cutscores)) {
 		if (is.character(performance.level.cutscores)) {
-			if (!(performance.level.cutscores %in% names(SGPstateData))) {
+			if (!(performance.level.cutscores %in% ls(SGPstateData))) {
 				tmp.messages <- c(tmp.messages, "\tNOTE: To use state cutscores, supply an appropriate two letter state abbreviation. \nRequested state may not be included. See help page for details.\n")
 				tf.cutscores <- FALSE
 			}
@@ -419,10 +451,6 @@ function(panel.data,	## REQUIRED
 		stop("Either percentile trajectories and/or performance level cutscores must be supplied for the analyses.")
 	}
 
-	if (missing(max.order.for.progression)) {
-		max.order.for.progression <- NULL
-	}
-
 	if (!is.null(achievement.level.prior.vname)) {
 		if (!achievement.level.prior.vname %in% names(panel.data[["Panel_Data"]])) {
 			tmp.messages <- c(tmp.messages, "\tNOTE: Supplied achievement.level.prior.vname is not in supplied panel.data. No ACHIEVEMENT_LEVEL_PRIOR variable will be produced.\n")
@@ -455,17 +483,16 @@ function(panel.data,	## REQUIRED
 		projcuts.digits <- 0
 	}
 
-	### Create ss.data from Panel_Data and rename variables in based upon grade.progression
 
-        ### Create ss.data from Panel_Data
+	### Create ss.data from Panel_Data and rename variables in based upon grade.progression
 
         if (!missing(panel.data.vnames)) {
                 if (!all(panel.data.vnames %in% names(panel.data[["Panel_Data"]]))) {
                         tmp.messages <- c(tmp.messages, "\tNOTE: Supplied 'panel.data.vnames' are not all in the supplied 'Panel_Data'.\n\t\tAnalyses will continue with the variables contained in both Panel_Data and those provided in the supplied argument 'panel.data.vnames'.\n")
                 }
-                ss.data <- panel.data[["Panel_Data"]][,intersect(panel.data.vnames, names(panel.data[["Panel_Data"]]))]
+                ss.data <- as.data.table(panel.data[["Panel_Data"]][,intersect(panel.data.vnames, names(panel.data[["Panel_Data"]]))])
         } else {
-                ss.data <- panel.data[["Panel_Data"]]
+                ss.data <- as.data.table(panel.data[["Panel_Data"]])
         }
 
 	if (dim(ss.data)[2] %% 2 != 1) {
@@ -485,7 +512,14 @@ function(panel.data,	## REQUIRED
 	num.predictors <- 1:length(grade.progression)
 	GD <- paste("GD", grade.progression, sep="")
 	SS <- paste("SS", grade.progression, sep="")
-	ss.data <- .get.data.table(ss.data)
+#	ss.data <- .get.data.table(ss.data)
+	setnames(ss.data, c(1, (1+num.panels-max(num.predictors)+1):(1+num.panels), (1+2*num.panels-max(num.predictors)+1):(1+2*num.panels)), c("ID", GD, SS))
+	setkey(ss.data, ID)
+
+
+
+	### Test to see if ss.data has cases to analyze
+
 	if (dim(.get.panel.data(ss.data, 1, by.grade, tmp.gp=grade.progression))[1] == 0) {
                 tmp.messages <- c(tmp.messages, "\tNOTE: Supplied data together with grade progression contains no data. Check data, function arguments and see help page for details.\n")
                 message(paste("\tStarted studentGrowthProjections", started.date))
@@ -503,35 +537,40 @@ function(panel.data,	## REQUIRED
                         Simulated_SGPs=panel.data[["Simulated_SGPs"]]))
         } 
 
-	### Get Coefficient_Matrices names
 
-	matrix.names <- names(panel.data[["Coefficient_Matrices"]][[tmp.path.coefficient.matrices]])
-	matrix.names <- matrix.names[sapply(strsplit(matrix.names, "_"), function(x) is.na(x[4]))] ## REMOVE names that have the grade.progression.label for now
+	### Calculate grade.projection.sequence, content_area.projection.sequence, and grade.projection.sequence.priors
 
-	### Calculate growth projections/trajectories 
+	tmp.matrices <- panel.data[["Coefficient_Matrices"]][[tmp.path.coefficient.matrices]]
 
-	max.grade <- .get.max.matrix.grade(matrix.names)
-	if (!missing(max.forward.progression.grade)) max.grade <- max.forward.progression.grade
 
-	if (tmp.last+1 > max.grade) {
-		stop("Supplied grade.progression and coefficient matrices do not allow projection. See help page for details.")
+	### Calculate grade.projection.sequence if not supplied
+
+	if (is.null(grade.projection.sequence)) {
+		grade.projection.sequence <- sort(unique(as.numeric(sapply(sapply(tmp.matrices, function(x) x@Grade_Progression), function(x) type.convert(tail(x, 1))))))
+		grade.projection.sequence <- grade.projection.sequence[grade.projection.sequence > max(grade.progression)]
+
+		if (identical(grade.projection.sequence, numeric(0))) {
+			stop("Supplied grade.progression and coefficient matrices do not allow projection. See help page for details.")
+		}
+
+		if (!missing(max.forward.progression.grade)) grade.projection.sequence <- grade.projection.sequence[grade.projection.sequence <= max.forward.progression.grade]
+		if (!missing(max.forward.progression.years)) grade.projection.sequence <- head(grade.projection.sequence, max.forward.progression.years)
 	}
 
-	if (!missing(max.forward.progression.years)) {
-		grade.projection.sequence <- (tmp.last+1):min(max.grade, tmp.last+1+max.forward.progression.years)
-		grade.projection.sequence <- grade.projection.sequence[grade.projection.sequence %in% as.numeric(unique(sapply(strsplit(matrix.names, "_"), function(x) x[2])))]
-	} else {
-		grade.projection.sequence <- (tmp.last+1):max.grade
-		grade.projection.sequence <- grade.projection.sequence[grade.projection.sequence %in% as.numeric(unique(sapply(strsplit(matrix.names, "_"), function(x) x[2])))]
+
+	### Calculate content_area.projection.sequence if not supplied
+
+	if (is.null(content_area.projection.sequence)) {
+		content_area.projection.sequence <- rep(sgp.labels[['my.subject']], length(grade.projection.sequence))
 	}
 
-	if (is.null(max.order.for.progression)) {
-		max.order.tf <- TRUE ## Use maximum order coefficient matrices by default
-	} else {
-		max.order.tf <- FALSE
-	}
-	
-	grade.projection.sequence.priors <- .get.grade.projection.sequence.priors(grade.progression, grade.projection.sequence, max.order.tf=max.order.tf) 
+
+	### Calculate grade.projection.sequence.priors 
+
+	grade.projection.sequence.priors <- get.grade.projection.sequence.priors(grade.progression, grade.projection.sequence) 
+
+
+	### Calculate percentile trajectories
 
 	percentile.trajectories <- .get.percentile.trajectories(ss.data)
 
@@ -546,8 +585,14 @@ function(panel.data,	## REQUIRED
 
 	trajectories.and.cuts <- .get.trajectories.and.cuts(percentile.trajectories, !is.null(percentile.trajectory.values), tf.cutscores, toupper(projection.unit))
 
-	if (is.null(SGProjections[[tmp.path]])) SGProjections[[tmp.path]] <- .unget.data.table(as.data.table(trajectories.and.cuts), ss.data)
-	else SGProjections[[tmp.path]] <- rbind.fill(SGProjections[[tmp.path]], .unget.data.table(as.data.table(trajectories.and.cuts), ss.data))
+	if (!is.null(achievement.level.prior.vname)) {
+		trajectories.and.cuts <- data.table(panel.data[["Panel_Data"]][,c("ID", achievement.level.prior.vname)], key="ID")[trajectories.and.cuts]
+		setnames(trajectories.and.cuts, achievement.level.prior.vname, "ACHIEVEMENT_LEVEL_PRIOR")
+	}
+
+#	SGProjections[[tmp.path]] <- rbind.fill(as.data.frame(SGProjections[[tmp.path]]), .unget.data.table(as.data.table(trajectories.and.cuts), ss.data))
+	SGProjections[[tmp.path]] <- rbind.fill(as.data.frame(SGProjections[[tmp.path]]), trajectories.and.cuts)
+
 
 	### Announce Completion & Return SGP Object
 
