@@ -1,7 +1,7 @@
 `visualizeSGP` <- 
 function(sgp_object,
 		plot.types=c("bubblePlot", "studentGrowthPlot", "growthAchievementPlot"),
-		state,
+		state=NULL,
 		bPlot.years=NULL,
 		bPlot.content_areas=NULL,
 		bPlot.districts=NULL,
@@ -23,6 +23,9 @@ function(sgp_object,
 		sgPlot.content_areas=NULL,
 		sgPlot.districts=NULL,
 		sgPlot.schools=NULL,
+		sgPlot.reports.by.school=TRUE,
+		sgPlot.instructors=NULL,
+		sgPlot.reports.by.instructor=FALSE,
 		sgPlot.students=NULL,
 		sgPlot.reports.by.student=FALSE,
 		sgPlot.header.footer.color="#4CB9CC",
@@ -35,6 +38,9 @@ function(sgp_object,
 		sgPlot.demo.report=FALSE,
 		sgPlot.produce.plots=TRUE,
 		sgPlot.baseline=NULL,
+		sgPlot.zip=TRUE,
+		sgPlot.output.format="PDF",
+		sgPlot.show.targets.years.forward=NULL,
 		gaPlot.years=NULL,
 		gaPlot.content_areas=NULL, 
 		gaPlot.students=NULL,
@@ -53,28 +59,28 @@ function(sgp_object,
 	ETHNICITY <- GENDER <- ID <- NULL ## To prevent R CMD check warnings
 	TEST_LEVEL <- SUBJECT_CODE <- SCALE_SCORE <- GRADE <- NULL ## To prevent R CMD check warnings
 	SCHOOL_ENROLLMENT_STATUS <- LAST_NAME <- FIRST_NAME <- NULL ## To prevent R CMD check warnings
-	MEDIAN_SGP <- MEDIAN_SGP_COUNT <- VALID_CASE <- gaPlot.iter <- sgPlot.iter <- V1 <- NULL ## To prevent R CMD check warnings
+	MEDIAN_SGP <- MEDIAN_SGP_COUNT <- VALID_CASE <- gaPlot.iter <- sgPlot.iter <- V1 <- variable <- INSTRUCTOR_NAME <- INSTRUCTOR_NUMBER <- NULL ## To prevent R CMD check warnings
+	CONTENT_AREA_RESPONSIBILITY <- NULL
 
 
 	### Create state (if missing) from sgp_object (if possible)
 
-	if (missing(state)) {
-		tmp.name <- gsub("_", " ", deparse(substitute(sgp_object)))
-		if (any(sapply(c(state.name, "Demonstration", "AOB"), function(x) regexpr(x, tmp.name))==1)) {
-			state <- c(state.abb, "DEMO", "AOB")[which(sapply(c(state.name, "Demonstration", "AOB"), function(x) regexpr(x, tmp.name))==1)]
-		}
-	}
+        if (is.null(state)) {
+                tmp.name <- toupper(gsub("_", " ", deparse(substitute(sgp_object))))
+                state <- getStateAbbreviation(tmp.name, "visualizeSGP")
+        }
+
 
 	### Set up parallel.config if NULL
 
 	if (is.null(parallel.config)) {
-		parallel.config = list(BACKEND="FOREACH", TYPE=NA, WORKERS=list(GA_PLOTS=1, SG_PLOTS=1))
+		parallel.config = list(BACKEND="FOREACH", TYPE=NA, WORKERS=list(GA_PLOTS=1, SG_PLOTS=1, SG_PLOTS_TARGETS=1))
 	}
 
 	### Utility functions	
 
 	"%w/o%" <- function(x,y) x[!x %in% y]
-	num_non_missing <- function(x) sum(!is.na(as.numeric(x)))
+	num_non_missing <- function(x) sum(!is.na(x))
 
 	.year.increment <- function(year, increment) {
 		paste(as.numeric(unlist(strsplit(as.character(year), "_")))+increment, collapse="_")
@@ -150,6 +156,12 @@ function(sgp_object,
 
 	} ### END get.gaPlot.iter
 
+	get.gaPlot.object <- function(sgp_object) {
+		tmp.sgp <- new("SGP")
+		tmp.sgp@Data <- sgp_object@Data
+		tmp.sgp@SGP <- sgp_object@SGP[c("Coefficient_Matrices", "Knots_Boundaries")]
+		return(tmp.sgp)
+	} ### END get.gaPlot.object
 
 ##############################################################################################################
 #### bubblePlot
@@ -202,13 +214,15 @@ function(sgp_object,
 		}
 
 		par.start <- startParallel(parallel.config, 'GA_PLOTS')
+
+		gaPlot.sgp_object <- get.gaPlot.object(sgp_object)
 		
 		if (par.start$par.type=="FOREACH") {
 
 			foreach(gaPlot.iter=iter(get.gaPlot.iter(gaPlot.years, gaPlot.content_areas, gaPlot.students)), .packages="SGP", .inorder=FALSE,
 				.options.multicore=par.start$foreach.options, .options.mpi=par.start$foreach.options, .options.redis=par.start$foreach.options) %dopar% {
 					growthAchievementPlot(
-						gaPlot.sgp_object=sgp_object,
+						gaPlot.sgp_object=gaPlot.sgp_object,
 						gaPlot.students=gaPlot.iter[["ID"]],
 						gaPlot.max.order.for.progression=get.max.order.for.progression(gaPlot.iter[["YEAR"]], gaPlot.iter[["CONTENT_AREA"]]),
 						state=state,
@@ -226,7 +240,7 @@ function(sgp_object,
 			gaPlot.list <- get.gaPlot.iter(gaPlot.years, gaPlot.content_areas, gaPlot.students)
 			clusterApplyLB(par.start$internal.cl, gaPlot.list, function(gaPlot.iter) 
 				growthAchievementPlot(
-						gaPlot.sgp_object=sgp_object,
+						gaPlot.sgp_object=gaPlot.sgp_object,
 						gaPlot.students=gaPlot.iter[["ID"]],
 						gaPlot.max.order.for.progression=get.max.order.for.progression(gaPlot.iter[["YEAR"]], gaPlot.iter[["CONTENT_AREA"]]),
 						state=state,
@@ -241,7 +255,7 @@ function(sgp_object,
 			gaPlot.list <- get.gaPlot.iter(gaPlot.years, gaPlot.content_areas, gaPlot.students)
 			mclapply(gaPlot.list, function(gaPlot.iter) {
 						growthAchievementPlot(
-						gaPlot.sgp_object=sgp_object,
+						gaPlot.sgp_object=gaPlot.sgp_object,
 						gaPlot.students=gaPlot.iter[["ID"]],
 						gaPlot.max.order.for.progression=get.max.order.for.progression(gaPlot.iter[["YEAR"]], gaPlot.iter[["CONTENT_AREA"]]),
 						state=state,
@@ -270,6 +284,15 @@ if ("studentGrowthPlot" %in% plot.types) {
 
 	#### Utility functions
 
+	get.next.grade <- function(grade, content_area) {
+		if (is.na(grade)) {
+			return(NA)
+		} else {
+			tmp.grades.reported <- SGPstateData[[state]][["Student_Report_Information"]][["Grades_Reported"]][[content_area]]
+			c(tmp.grades.reported, NA)[match(grade, tmp.grades.reported)+1]
+		}
+	}
+
 	get.my.label <- function(state, content_area, year, label="Cutscores") {
 		tmp.cutscore.years <- sapply(strsplit(names(SGPstateData[[state]][["Achievement"]][[label]])[grep(content_area, names(SGPstateData[[state]][["Achievement"]][[label]]))], "[.]"),
                         function(x) x[2])
@@ -308,7 +331,7 @@ if ("studentGrowthPlot" %in% plot.types) {
 	get.years.content_areas.grades <- function(state) {
 		tmp.list <- list()
 		for (i in names(SGPstateData[[state]][["Student_Report_Information"]][["Grades_Reported"]])) {
-			tmp.df <- data.frame(GRADE=SGPstateData[[state]][["Student_Report_Information"]][["Grades_Reported"]][[i]])
+			tmp.df <- data.frame(GRADE=as.character(SGPstateData[[state]][["Student_Report_Information"]][["Grades_Reported"]][[i]]), stringsAsFactors=FALSE)
 			if (!is.null(SGPstateData[[state]][["Student_Report_Information"]][["Earliest_Year_Reported"]][[i]])) {
 				tmp.df <- CJ(tmp.df$GRADE, SGPstateData[[state]][["Student_Report_Information"]][["Earliest_Year_Reported"]][[i]]:tmp.last.year)
 			} else {
@@ -329,9 +352,16 @@ if ("studentGrowthPlot" %in% plot.types) {
 
 	#### Some checks
 
-		if (!is.null(sgPlot.students) | !is.null(sgPlot.schools) | !is.null(sgPlot.districts)) sgPlot.demo.report <- FALSE
+		if (!is.null(sgPlot.students) | !is.null(sgPlot.instructors) | !is.null(sgPlot.schools) | !is.null(sgPlot.districts)) sgPlot.demo.report <- FALSE
+		if (!is.null(sgPlot.students)) sgPlot.reports.by.student <- TRUE
+		if (!is.null(sgPlot.instructors)) sgPlot.reports.by.instructor <- TRUE
+		if (!is.null(sgPlot.schools)) sgPlot.reports.by.school <- TRUE
+		if (!is.null(sgPlot.districts)) sgPlot.reports.by.school <- TRUE
 		if (is.data.frame(sgp_object)) sgPlot.wide.data <- TRUE
-		if (is.SGP(sgp_object)) sgPlot.wide.data <- FALSE
+		if (is.SGP(sgp_object)) {
+			sgPlot.wide.data <- FALSE
+			slot.data <- copy(sgp_object@Data)
+		}
 
 		if (sgPlot.demo.report & sgPlot.wide.data) {
 			message("\tNOTE: Demonstration report is not supported using wide data. Process will proceed with demonstration report production using long data.\n")
@@ -348,13 +378,24 @@ if ("studentGrowthPlot" %in% plot.types) {
 		}
 
 		if (sgPlot.baseline) {
-			 my.sgp <- "SGP_BASELINE"
-			 my.sgp.level <- "SGP_LEVEL_BASELINE"
+			my.sgp <- "SGP_BASELINE"
+			my.sgp.level <- "SGP_LEVEL_BASELINE"
+			if (!is.null(sgPlot.show.targets.years.forward)) my.target.types <- c("sgp.projections.baseline", "sgp.projections.lagged.baseline")
 		} else {
-			 my.sgp <- "SGP"
-			 my.sgp.level <- "SGP_LEVEL"
+			my.sgp <- "SGP"
+			my.sgp.level <- "SGP_LEVEL"
+			if (!is.null(sgPlot.show.targets.years.forward)) my.target.types <- c("sgp.projections", "sgp.projections.lagged")
 		}
 
+	#### Which targets
+
+		if (length(which(SGPstateData[[state]][["Achievement"]][["Levels"]][["Proficient"]]=="Proficient")) <= 1) {
+			my.target.levels <- "CATCH_UP_KEEP_UP"
+		} else {
+			my.target.levels <- c("CATCH_UP_KEEP_UP", "MOVE_UP_STAY_UP")
+		}
+
+
 ################################################
 ######## IF sgPlot.wide.data is supplied 
 ################################################
@@ -364,8 +405,8 @@ if (sgPlot.wide.data) { ### When WIDE data is provided
 	#### Calculate years and content area from data
  
 		tmp.all.years <- sort(unique(sapply(strsplit(names(sgp_object), "[.]"), function(x) x[2])))
-		tmp.years <- type.convert(tail(tmp.all.years, 5))
-		tmp.last.year <- type.convert(tail(tmp.all.years, 1))
+		tmp.years <- tail(tmp.all.years, 5)
+		tmp.last.year <- tail(tmp.all.years, 1)
 		tmp.content_areas <- unique(sgp_object[["CONTENT_AREA"]])
 
 	#### Reconcile School and District selections
@@ -430,11 +471,10 @@ if (sgPlot.wide.data) { ### When WIDE data is provided
 			setkeyv(sgp_object, c("CONTENT_AREA", paste("SCHOOL_NUMBER", tmp.last.year, sep=".")))
 			tmp.ids <- unique(sgp_object[CJ(tmp.content_areas, tmp.districts.and.schools[["SCHOOL_NUMBER"]]), nomatch=0][["ID"]])
 			setkey(sgp_object, ID)
-			sgPlot.data <- sgp_object[tmp.ids]
+			sgPlot.data <- sgp_object[list(tmp.ids)]
 		} else { ## sgPlot.students specified with WIDE data
 			setkey(sgp_object, ID)
-			if (is.factor(sgp_object$ID)) sgPlot.students <- as.factor(sgPlot.students)
-			sgPlot.data <- sgp_object[sgPlot.students]
+			sgPlot.data <- sgp_object[list(sgPlot.students)]
 			tmp.districts.and.schools <- unique(data.table(sgPlot.data, 
 				key=paste(c("DISTRICT_NUMBER", "SCHOOL_NUMBER"), tmp.last.year, sep=".")))[,
 				c(paste(c("DISTRICT_NUMBER", "SCHOOL_NUMBER"), tmp.last.year, sep=".")), with=FALSE]
@@ -448,36 +488,18 @@ if (sgPlot.wide.data) { ### When WIDE data is provided
 ######## IF WIDE data is NOT supplied
 ###############################################
 
-	#### Some data prep
-
-	if (!is.null(sgPlot.districts)) {
-		if (is.factor(sgp_object@Data$DISTRICT_NUMBER)) {
-			sgPlot.districts <- as.factor(sgPlot.districts)
-		} else {
-			sgPlot.districts <- as.integer(sgPlot.districts)
-		}
-	}
-
-	if (!is.null(sgPlot.schools)) {
-		if (is.factor(sgp_object@Data$SCHOOL_NUMBER)) {
-			sgPlot.schools <- as.factor(sgPlot.schools)
-		} else {
-			sgPlot.schools <- as.integer(sgPlot.schools)
-		}
-	}
-
 	#### Set key on LONG data
 
 	long.key <- c("VALID_CASE", "YEAR", "CONTENT_AREA", "DISTRICT_NUMBER", "SCHOOL_NUMBER")
-	setkeyv(sgp_object@Data, long.key)
+	setkeyv(slot.data, long.key)
 
 	#### Year stuff (NECESSARY even IF WIDE data is provided)
 
 	if (is.null(sgPlot.years)) {
-		tmp.years <- tail(sort(unique(sgp_object@Data["VALID_CASE"][["YEAR"]])), 5)
+		tmp.years <- tail(sort(unique(slot.data["VALID_CASE"][["YEAR"]])), 5)
 		tmp.last.year <- tail(tmp.years, 1)
 	} else {
-		tmp.all.years <- sort(unique(sgp_object@Data["VALID_CASE"][["YEAR"]])) 
+		tmp.all.years <- sort(unique(slot.data["VALID_CASE"][["YEAR"]])) 
 		tmp.years <- tail(tmp.all.years[1:which(tmp.all.years==tail(sort(sgPlot.years), 1))], 5)
 		tmp.last.year <- tail(tmp.years, 1)
 	}
@@ -486,12 +508,47 @@ if (sgPlot.wide.data) { ### When WIDE data is provided
 
 	if (is.null(sgPlot.content_areas)) {
 		tmp.content_areas <- sort(intersect(
-						unique(sgp_object@Data[SJ("VALID_CASE", tmp.last.year), nomatch=0][["CONTENT_AREA"]]),
+						unique(slot.data[SJ("VALID_CASE", tmp.last.year), nomatch=0][["CONTENT_AREA"]]),
 						names(SGPstateData[[state]][["Student_Report_Information"]][["Content_Areas_Labels"]])))
 	} else {
 		tmp.content_areas <- sgPlot.content_areas
 	}
 
+
+	#### Melt and rename data if reporting by instructor number
+
+	if (sgPlot.reports.by.instructor) {
+		instructor.variable.names <- grep("INSTRUCTOR_NUMBER", names(slot.data), value=TRUE)
+		district.and.school.variable.names <- c("VALID_CASE", "CONTENT_AREA", "YEAR", "DISTRICT_NUMBER", "SCHOOL_NUMBER")
+		student.teacher.lookup <- data.table(melt(as.data.frame(slot.data[VALID_CASE=="VALID_CASE" & YEAR==tmp.last.year, 
+			c(district.and.school.variable.names, "ID", instructor.variable.names), with=FALSE]),
+                                        measure.vars=grep("INSTRUCTOR_NUMBER", names(slot.data), value=TRUE),
+                                        value.name="INSTRUCTOR_NUMBER"))
+		invisible(student.teacher.lookup[,variable:=NULL])
+		if (length(grep("INSTRUCTOR_FIRST_NAME", names(slot.data))) > 0) {
+			invisible(student.teacher.lookup[, INSTRUCTOR_NAME:=paste(
+				melt(as.data.frame(slot.data[VALID_CASE=="VALID_CASE" & YEAR==tmp.last.year, grep("INSTRUCTOR_LAST_NAME", names(slot.data), value=TRUE), with=FALSE]),
+				measure.vars=grep("INSTRUCTOR_LAST_NAME", names(slot.data), value=TRUE))[,2],
+				melt(as.data.frame(slot.data[VALID_CASE=="VALID_CASE" & YEAR==tmp.last.year, grep("INSTRUCTOR_FIRST_NAME", names(slot.data), value=TRUE), with=FALSE]),
+				measure.vars=grep("INSTRUCTOR_FIRST_NAME", names(slot.data), value=TRUE))[,2], sep=", ")])
+			invisible(student.teacher.lookup[INSTRUCTOR_NAME=="NA, NA", INSTRUCTOR_NAME := as.character(NA)])
+		} else {
+			invisible(student.teacher.lookup[!is.na(INSTRUCTOR_NUMBER), INSTRUCTOR_NAME := paste("Instructor", INSTRUCTOR_NUMBER)])
+		}
+		if (!is.null(sgPlot.instructors)) {
+			student.teacher.lookup <- subset(student.teacher.lookup, INSTRUCTOR_NUMBER %in% sgPlot.instructors)
+		} else {
+			student.teacher.lookup <- subset(student.teacher.lookup, !is.na(INSTRUCTOR_NUMBER))
+		}
+		tmp.district.and.schools.instructors <- unique(data.table(student.teacher.lookup, key=long.key)[,district.and.school.variable.names, with=FALSE])
+		student.teacher.lookup <- student.teacher.lookup[,list(ID, CONTENT_AREA, INSTRUCTOR_NUMBER, INSTRUCTOR_NAME)]
+		invisible(student.teacher.lookup[,CONTENT_AREA_RESPONSIBILITY:=factor(1, levels=0:1, labels=c("Content Area Responsibility: No", "Content Area Responsibility: Yes"))])
+		setnames(student.teacher.lookup, c("INSTRUCTOR_NUMBER", "INSTRUCTOR_NAME"), paste(c("INSTRUCTOR_NUMBER", "INSTRUCTOR_NAME"), tmp.last.year, sep="."))
+	}
+
+
+
+
 	#### The following apply when sgPlot.students is not supplied 
 
 	if (is.null(sgPlot.students)) {
@@ -501,77 +558,82 @@ if (sgPlot.wide.data) { ### When WIDE data is provided
 		if (sgPlot.demo.report) {
 			sgPlot.anonymize <- TRUE
 			tmp.ids <- list()
-			setkeyv(sgp_object@Data, c("VALID_CASE", "YEAR", "GRADE"))
+			setkeyv(slot.data, c("VALID_CASE", "YEAR", "GRADE"))
 			tmp.grades.reported <- unique(unlist(SGPstateData[[state]][["Student_Report_Information"]][["Grades_Reported"]]))
-			tmp.grades.reported <- tmp.grades.reported[tmp.grades.reported %in% unique(sgp_object@Data)["VALID_CASE"][["GRADE"]]]
+			tmp.grades.reported <- as.character(tmp.grades.reported[tmp.grades.reported %in% unique(slot.data)["VALID_CASE"][["GRADE"]]])
 			for (i in seq_along(tmp.grades.reported)) {
-				tmp.ids[[i]] <- as.character(sample(unique(sgp_object@Data[SJ("VALID_CASE", tmp.last.year, tmp.grades.reported[i])]$ID), 10))
+				tmp.ids[[i]] <- as.character(sample(unique(slot.data[SJ("VALID_CASE", tmp.last.year, tmp.grades.reported[i])]$ID), 10))
 			}
-			sgp_object@Data$SCHOOL_NUMBER <- as.integer(sgp_object@Data$SCHOOL_NUMBER)
-			sgp_object@Data$SCHOOL_NUMBER[sgp_object@Data$ID %in% unlist(tmp.ids)] <- -99L
-			sgp_object@Data$DISTRICT_NUMBER <- as.integer(sgp_object@Data$DISTRICT_NUMBER)
-			sgp_object@Data$DISTRICT_NUMBER[sgp_object@Data$ID %in% unlist(tmp.ids)] <- -999L
+			slot.data[,c("SCHOOL_NUMBER", "DISTRICT_NUMBER") := NULL]
+			slot.data[slot.data$ID %in% unlist(tmp.ids), c("SCHOOL_NUMBER", "DISTRICT_NUMBER") := list(-99L, -999L)]
 			tmp.districts.and.schools <- CJ("VALID_CASE", tmp.last.year, tmp.content_areas, -999L, -99L)
 			setnames(tmp.districts.and.schools, c("VALID_CASE", "YEAR", "CONTENT_AREA", "DISTRICT_NUMBER", "SCHOOL_NUMBER"))
 			setkeyv(tmp.districts.and.schools, long.key) 
-			setkeyv(sgp_object@Data, long.key)
+			setkeyv(slot.data, long.key)
 		} else {
 
 		#### Reconcile School and District selections
 
-			setkeyv(sgp_object@Data, c("VALID_CASE", "YEAR", "DISTRICT_NUMBER", "SCHOOL_NUMBER"))
-			tmp.districts.and.schools.size <- sgp_object@Data[SJ("VALID_CASE", tmp.last.year)][,num_non_missing(ID), by=list(DISTRICT_NUMBER, SCHOOL_NUMBER)]
-			setkeyv(tmp.districts.and.schools.size, c("DISTRICT_NUMBER", "SCHOOL_NUMBER"))
-			setkeyv(sgp_object@Data, long.key)
+			if (sgPlot.reports.by.instructor) {
+				tmp.districts.and.schools <- tmp.district.and.schools.instructors
+			} else {
+				setkeyv(slot.data, long.key)
 
-			if (is.null(sgPlot.schools) | is.null(sgPlot.districts)) {
-				tmp.districts.and.schools <- unique(data.table(sgp_object@Data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas)][,
-					list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER)], 
-					key=key(sgp_object)))
-			} 
-			if (is.null(sgPlot.schools) & !is.null(sgPlot.districts)) {
-				tmp.districts.and.schools <- unique(data.table(sgp_object@Data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas, sgPlot.districts)][,
-					list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER)], 
-					key=key(sgp_object)))
-			}
-			if (!is.null(sgPlot.schools) & is.null(sgPlot.districts)) {
-				setkeyv(sgp_object@Data, c("VALID_CASE", "YEAR", "CONTENT_AREA", "SCHOOL_NUMBER", "DISTRICT_NUMBER"))
-				tmp.districts.and.schools <- unique(data.table(sgp_object@Data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas, sgPlot.schools)][,
-					list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER)], 
-					key=long.key))
-				setkeyv(sgp_object@Data, long.key)
-			}
-			if (!is.null(sgPlot.schools) & !is.null(sgPlot.districts)) {
-				tmp.list <- list()
-				for (i in seq_along(sgPlot.districts)) {
-					tmp.schools <- unique(sgp_object@Data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas, sgPlot.districts[i])][["SCHOOL_NUMBER"]])
-					if (any(sgPlot.schools==tmp.schools)) {
-						tmp.list[[i]] <- tmp.schools[sgPlot.schools==tmp.schools]
-					} else {
-						tmp.list[[i]] <- tmp.schools
-					}
-				}
-				sgPlot.schools <- unique(c(sgPlot.schools, do.call(c, tmp.list)))
-				setkeyv(sgp_object@Data, c("VALID_CASE", "YEAR", "CONTENT_AREA", "SCHOOL_NUMBER", "DISTRICT_NUMBER"))
-				tmp.districts.and.schools <- unique(data.table(sgp_object@Data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas, sgPlot.schools)][,
+				if (is.null(sgPlot.schools) | is.null(sgPlot.districts)) {
+					tmp.districts.and.schools <- unique(data.table(slot.data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas)][,
 						list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER)], 
 						key=long.key))
-				setkeyv(sgp_object@Data, long.key)
-			} 
-		}
+				} 
+				if (is.null(sgPlot.schools) & !is.null(sgPlot.districts)) {
+					tmp.districts.and.schools <- unique(data.table(slot.data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas, sgPlot.districts)][,
+						list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER)], 
+						key=long.key))
+				}
+				if (!is.null(sgPlot.schools) & is.null(sgPlot.districts)) {
+					setkeyv(slot.data, c("VALID_CASE", "YEAR", "CONTENT_AREA", "SCHOOL_NUMBER", "DISTRICT_NUMBER"))
+					tmp.districts.and.schools <- unique(data.table(slot.data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas, sgPlot.schools)][,
+						list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER)], 
+						key=long.key))
+					setkeyv(slot.data, long.key)
+				}
+				if (!is.null(sgPlot.schools) & !is.null(sgPlot.districts)) {
+					tmp.list <- list()
+					for (i in seq_along(sgPlot.districts)) {
+						tmp.schools <- unique(slot.data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas, sgPlot.districts[i])][["SCHOOL_NUMBER"]])
+						if (any(sgPlot.schools==tmp.schools)) {
+							tmp.list[[i]] <- tmp.schools[sgPlot.schools==tmp.schools]
+						} else {
+							tmp.list[[i]] <- tmp.schools
+						}
+					}
+					sgPlot.schools <- unique(c(sgPlot.schools, do.call(c, tmp.list)))
+					setkeyv(slot.data, c("VALID_CASE", "YEAR", "CONTENT_AREA", "SCHOOL_NUMBER", "DISTRICT_NUMBER"))
+					tmp.districts.and.schools <- unique(data.table(slot.data[CJ("VALID_CASE", tmp.last.year, tmp.content_areas, sgPlot.schools)][,
+							list(VALID_CASE, YEAR, CONTENT_AREA, DISTRICT_NUMBER, SCHOOL_NUMBER)], 
+							key=long.key))
+					setkeyv(slot.data, long.key)
+				}
+			}  ## END if else (sgPlot.report.by.instructors)
+		} ## END if else (sgPlot.demo.report)
 	} ## END if (is.null(sgPlot.students))
 
 	#### Subset data (NOT NECESSARY IF WIDE data is provided)
 
+		setkeyv(slot.data, c("VALID_CASE", "YEAR", "DISTRICT_NUMBER", "SCHOOL_NUMBER"))
+		tmp.districts.and.schools.size <- slot.data[SJ("VALID_CASE", tmp.last.year)][,num_non_missing(ID), by=list(DISTRICT_NUMBER, SCHOOL_NUMBER)]
+		setkeyv(tmp.districts.and.schools.size, c("DISTRICT_NUMBER", "SCHOOL_NUMBER"))
+		setkeyv(slot.data, long.key)
+
 		if (is.null(sgPlot.students)) {
-			report.ids <- unique(sgp_object@Data[tmp.districts.and.schools][["ID"]])
-			setkeyv(sgp_object@Data, c("CONTENT_AREA", "GRADE", "YEAR"))
-			tmp.table <- data.table(sgp_object@Data[get.years.content_areas.grades(state)], 
+			report.ids <- unique(slot.data[tmp.districts.and.schools][["ID"]])
+			if (sgPlot.reports.by.instructor) report.ids <- intersect(student.teacher.lookup[['ID']], report.ids)
+			setkeyv(slot.data, c("CONTENT_AREA", "GRADE", "YEAR"))
+			tmp.table <- data.table(slot.data[get.years.content_areas.grades(state)], 
 				key=c("ID", "CONTENT_AREA", "YEAR", "VALID_CASE"))[CJ(report.ids, tmp.content_areas, tmp.years, "VALID_CASE")]
 		} else {
 			report.ids <- sgPlot.students
-			setkeyv(sgp_object@Data, c("CONTENT_AREA", "GRADE", "YEAR"))
-			tmp.table <- data.table(sgp_object@Data[get.years.content_areas.grades(state)], 
+			setkeyv(slot.data, c("CONTENT_AREA", "GRADE", "YEAR"))
+			tmp.table <- data.table(slot.data[get.years.content_areas.grades(state)], 
 				key=c("VALID_CASE", "ID", "CONTENT_AREA", "YEAR"))[CJ("VALID_CASE", report.ids, tmp.content_areas, tmp.years)]
 			setkeyv(tmp.table, c("VALID_CASE", "YEAR", "CONTENT_AREA", "DISTRICT_NUMBER", "SCHOOL_NUMBER"))
 			tmp.districts.and.schools <- tmp.table[CJ("VALID_CASE", tmp.last.year, tmp.content_areas)][, list(DISTRICT_NUMBER, SCHOOL_NUMBER)]
@@ -583,6 +645,7 @@ if (sgPlot.wide.data) { ### When WIDE data is provided
 		tmp.districts.and.schools <- subset(tmp.districts.and.schools, !is.na(DISTRICT_NUMBER) & !is.na(SCHOOL_NUMBER))
 		if (!sgPlot.demo.report) tmp.districts.and.schools <- tmp.districts.and.schools.size[tmp.districts.and.schools][order(V1, decreasing=TRUE)][,list(DISTRICT_NUMBER, SCHOOL_NUMBER)]
 
+
 	#### Invalidate bad SGPs (if necessary)
 
 		if ("VALID_SGP" %in% names(tmp.table)) {
@@ -609,8 +672,8 @@ if (sgPlot.wide.data) { ### When WIDE data is provided
 			setkey(tmp.dt, ID)
 			tmp.dt <- tmp.dt[!duplicated(tmp.dt),]
 
-			tmp.dt$LAST_NAME <- randomNames(gender=tmp.dt$GENDER, ethnicity=tmp.dt$ETHNICITY, which.names="last")
-			tmp.dt$FIRST_NAME <- randomNames(gender=tmp.dt$GENDER, ethnicity=tmp.dt$ETHNICITY, which.names="first")
+			tmp.dt[,LAST_NAME := randomNames(gender=tmp.dt$GENDER, ethnicity=tmp.dt$ETHNICITY, which.names="last")]
+			tmp.dt[,FIRST_NAME := randomNames(gender=tmp.dt$GENDER, ethnicity=tmp.dt$ETHNICITY, which.names="first")]
 
 			names.dt <- tmp.dt[,list(ID, LAST_NAME, FIRST_NAME)]
 			setkey(names.dt, ID)
@@ -618,22 +681,22 @@ if (sgPlot.wide.data) { ### When WIDE data is provided
 			setkey(tmp.table, ID)
 			tmp.table <- names.dt[tmp.table]
 			if (sgPlot.demo.report) {
-				tmp.table$DISTRICT_NAME <- as.factor("Sample District")
-				tmp.table$SCHOOL_NAME <- as.factor("Sample School")
+				tmp.table[,DISTRICT_NAME := as.factor("Sample District")]
+				tmp.table[,SCHOOL_NAME := as.factor("Sample School")]
 			} else {
 				setkey(tmp.table, DISTRICT_NUMBER)
 				tmp.district.number <- data.table(DISTRICT_NUMBER=unique(tmp.table$DISTRICT_NUMBER) %w/o% NA, seq_along(unique(tmp.table$DISTRICT_NUMBER) %w/o% NA), 
-					key="DISTRICT_NUMBER")[tmp.table]$V2
-				tmp.table$DISTRICT_NAME <- as.character(tmp.table$DISTRICT_NAME)
-				tmp.table$DISTRICT_NAME[!is.na(tmp.table$DISTRICT_NUMBER)] <- paste("Sample District", tmp.district.number[!is.na(tmp.table$DISTRICT_NUMBER)])
-				tmp.table$DISTRICT_NAME <- as.factor(tmp.table$DISTRICT_NAME)
+					key="DISTRICT_NUMBER")[tmp.table][['V2']]
+				tmp.table[, DISTRICT_NAME := as.character(tmp.table$DISTRICT_NAME)]
+				tmp.table[!is.na(tmp.table$DISTRICT_NUMBER), DISTRICT_NAME := paste("Sample District", tmp.district.number[!is.na(tmp.table$DISTRICT_NUMBER)])]
+				tmp.table[, DISTRICT_NAME := as.factor(tmp.table$DISTRICT_NAME)]
 	
 				setkey(tmp.table, SCHOOL_NUMBER)
 				tmp.school.number <- data.table(SCHOOL_NUMBER=unique(tmp.table$SCHOOL_NUMBER) %w/o% NA, seq_along(unique(tmp.table$SCHOOL_NUMBER) %w/o% NA), 
 					key="SCHOOL_NUMBER")[tmp.table]$V2
-				tmp.table$SCHOOL_NAME <- as.character(tmp.table$SCHOOL_NAME)
-				tmp.table$SCHOOL_NAME[!is.na(tmp.table$SCHOOL_NUMBER)] <- paste("Sample School", tmp.school.number[!is.na(tmp.table$SCHOOL_NUMBER)])
-				tmp.table$SCHOOL_NAME <- as.factor(tmp.table$SCHOOL_NAME)
+				tmp.table[SCHOOL_NAME := as.character(tmp.table$SCHOOL_NAME)]
+				tmp.table[!is.na(tmp.table$SCHOOL_NUMBER), SCHOOL_NAME := paste("Sample School", tmp.school.number[!is.na(tmp.table$SCHOOL_NUMBER)])]
+				tmp.table[, SCHOOL_NAME := as.factor(tmp.table$SCHOOL_NAME)]
 			}
 		} ## END if (sgPlot.anonymize)
 
@@ -676,27 +739,90 @@ if (sgPlot.wide.data) { ### When WIDE data is provided
 				tmp.list[[i]] <- data.table(CONTENT_AREA=unlist(strsplit(i, "[.]"))[1],
 					sgp_object@SGP[["SGProjections"]][[i]][,c(1, grep("PROJ_YEAR_1", names(sgp_object@SGP[["SGProjections"]][[i]])))])
 			}
-			sgPlot.data <- data.table(rbind.fill(tmp.list), key=paste(key(sgPlot.data), collapse=","))[sgPlot.data]
+			sgPlot.data <- data.table(rbindlist(tmp.list), key=key(sgPlot.data))[sgPlot.data]
 			tmp.grade.name <- paste("GRADE", tmp.last.year, sep=".")
 			tmp.year.name <- .year.increment(tmp.last.year, 1)
 			setkeyv(sgPlot.data, c("CONTENT_AREA", tmp.grade.name))
 			for (proj.iter in grep("PROJ_YEAR_1", names(sgPlot.data))) {
 				tmp.scale_score.name <- names(sgPlot.data)[proj.iter]
-				sgPlot.data[[proj.iter]] <- sgPlot.data[,piecewise.transform(get(tmp.scale_score.name), state, CONTENT_AREA, tmp.year.name, get(tmp.grade.name)[1]+1), 
-					by=list(CONTENT_AREA, sgPlot.data[[tmp.grade.name]])]$V1 
+				sgPlot.data[[proj.iter]] <- sgPlot.data[,piecewise.transform(get(tmp.scale_score.name), state, CONTENT_AREA, tmp.year.name, get.next.grade(get(tmp.grade.name)[1], CONTENT_AREA[1])), 
+					by=list(CONTENT_AREA, sgPlot.data[[tmp.grade.name]])][['V1']] 
 			}
+		} ### END if (sgPlot.baseline)
+
+
+
+
+
+##############################################################################################################
+	#### BEGIN CONSTRUCTION ZONE: Calculate and Merge in lagged targets if requested
+##############################################################################################################
+
+		if (!is.null(sgPlot.show.targets.years.forward)) {
+
+			setkey(sgPlot.data, ID, CONTENT_AREA)
+
+			for (target.type in my.target.types) {
+				for (target.level in my.target.levels) {
+					sgPlot.data <- data.table(getTargetSGP(sgp_object, tmp.content_areas, state, tmp.last.year, 
+						target.type, target.level, sgPlot.show.targets.years.forward, unique(sgPlot.data[['ID']]), FALSE), key=c("ID", "CONTENT_AREA"))[sgPlot.data]
+					invisible(sgPlot.data[,VALID_CASE := NULL]); invisible(sgPlot.data[,YEAR := NULL])
+				}
+			} 
+
+
+browser()
+
+			for (target.type in my.target.types) {
+				for (target.level in my.target.levels) {
+					my.sgp.target <- getTargetName(target.type, target.level, sgPlot.show.targets.years.forward)
+					getTargetScaleScore(
+						sgp_object, 
+						state, 
+						sgPlot.data[, c("ID", my.sgp.target), with=FALSE], 
+						target.type, 
+						target.level, 
+						subset(get.years.content_areas.grades(state), YEAR==tmp.last.year),
+						parallel.config=parallel.config)
+
+				}
+			}
+
+		} ### if (!is.null(sgPlot.show.targets.years.forward))
+
+
+##############################################################################################################
+	#### END CONSTRUCTION ZONE: Calculate and Merge in lagged targets if requested
+##############################################################################################################
+
+
+
+
+	### Merge in INSTRUCTOR_NAME if requested
+
+		if (sgPlot.reports.by.instructor) {
+			setkeyv(student.teacher.lookup, c("ID", paste("INSTRUCTOR_NUMBER", tmp.last.year, sep=".")))
+			unique.teacher.lookup <- unique(student.teacher.lookup)[,c("ID", paste(c("INSTRUCTOR_NUMBER", "INSTRUCTOR_NAME"), tmp.last.year, sep=".")), with=FALSE]
+			setkey(sgPlot.data, ID); setkey(unique.teacher.lookup)
+			sgPlot.data <- sgPlot.data[unique.teacher.lookup]
+			tmp.key <- c("ID", paste("INSTRUCTOR_NUMBER", tmp.last.year, sep="."), "CONTENT_AREA")
+			setkeyv(sgPlot.data, tmp.key)
+			sgPlot.data <- data.table(student.teacher.lookup[,c(tmp.key, "CONTENT_AREA_RESPONSIBILITY"), with=FALSE], key=tmp.key)[sgPlot.data]
+			sgPlot.data[['CONTENT_AREA_RESPONSIBILITY']][is.na( sgPlot.data[['CONTENT_AREA_RESPONSIBILITY']])] <- "Content Area Responsibility: No"
 		}
 
-		setkeyv(sgp_object@Data, c("VALID_CASE", "CONTENT_AREA", "YEAR", "ID"))
+		### Rekey @Data
+
+		setkeyv(slot.data, c("VALID_CASE", "CONTENT_AREA", "YEAR", "ID"))
 
 } ## END if else (sgPlot.wide.data)
 
 
-#### Save WIDE file if requested
+#### Save WIDE file is requested
 
 if (sgPlot.save.sgPlot.data) {
 	setkey(sgPlot.data, ID)
-	tmp.file.name <- paste(c(gsub(" ", "_", state.name), "Demonstration")[which(state==c(state.abb, "DEMO"))], "ISR_Data", sep="_")
+	tmp.file.name <- paste(c(gsub(" ", "_", state.name), "Demonstration")[which(state==c(state.abb, "DEMO"))], "studentGrowthPlot_Data", sep="_")
 	assign(tmp.file.name, sgPlot.data)
 	save(list=tmp.file.name, file=paste(tmp.file.name, ".Rdata", sep=""))
 }
@@ -715,6 +841,8 @@ if (sgPlot.produce.plots) {
 			districts=tmp.districts.and.schools[["DISTRICT_NUMBER"]],
 			schools=tmp.districts.and.schools[["SCHOOL_NUMBER"]],
 			reports.by.student=sgPlot.reports.by.student,
+			reports.by.instructor=sgPlot.reports.by.instructor,
+			reports.by.school=sgPlot.reports.by.school,
 			sgPlot.years=tmp.years,
 			sgPlot.folder=sgPlot.folder,
 			sgPlot.folder.names=sgPlot.folder.names,
@@ -724,7 +852,9 @@ if (sgPlot.produce.plots) {
 			sgPlot.header.footer.color=sgPlot.header.footer.color,
 			sgPlot.fan=sgPlot.fan,
 			sgPlot.cleanup=sgPlot.cleanup,
-			sgPlot.baseline=sgPlot.baseline)
+			sgPlot.baseline=sgPlot.baseline,
+			sgPlot.zip=sgPlot.zip,
+			sgPlot.output.format=sgPlot.output.format)
 
 	} else { ### Parallel Processing
 		
@@ -743,6 +873,8 @@ if (sgPlot.produce.plots) {
 							districts=sgPlot.iter[["DISTRICT_NUMBER"]],
 							schools=sgPlot.iter[["SCHOOL_NUMBER"]],
 							reports.by.student=sgPlot.reports.by.student,
+							reports.by.instructor=sgPlot.reports.by.instructor,
+							reports.by.school=sgPlot.reports.by.school,
 							sgPlot.years=tmp.years,
 							sgPlot.folder=sgPlot.folder,
 							sgPlot.folder.names=sgPlot.folder.names,
@@ -752,7 +884,9 @@ if (sgPlot.produce.plots) {
 							sgPlot.header.footer.color=sgPlot.header.footer.color,
 							sgPlot.fan=sgPlot.fan,
 							sgPlot.cleanup=sgPlot.cleanup,
-							sgPlot.baseline=sgPlot.baseline)
+							sgPlot.baseline=sgPlot.baseline,
+							sgPlot.zip=sgPlot.zip,
+							sgPlot.output.format=sgPlot.output.format)
 			} ### END dopar
 		} ### END if FOREACH
 		
@@ -768,6 +902,8 @@ if (sgPlot.produce.plots) {
 					districts=sgPlot.iter[["DISTRICT_NUMBER"]],
 					schools=sgPlot.iter[["SCHOOL_NUMBER"]],
 					reports.by.student=sgPlot.reports.by.student,
+					reports.by.instructor=sgPlot.reports.by.instructor,
+					reports.by.school=sgPlot.reports.by.school,
 					sgPlot.years=tmp.years,
 					sgPlot.folder=sgPlot.folder,
 					sgPlot.folder.names=sgPlot.folder.names,
@@ -777,7 +913,9 @@ if (sgPlot.produce.plots) {
 					sgPlot.header.footer.color=sgPlot.header.footer.color,
 					sgPlot.fan=sgPlot.fan,
 					sgPlot.cleanup=sgPlot.cleanup,
-					sgPlot.baseline=sgPlot.baseline))
+					sgPlot.baseline=sgPlot.baseline,
+					sgPlot.zip=sgPlot.zip,
+					sgPlot.output.format=sgPlot.output.format))
 		} ### END if SNOW
 		
 		if (par.start$par.type=="MULTICORE") {
@@ -792,6 +930,8 @@ if (sgPlot.produce.plots) {
 					districts=sgPlot.iter[["DISTRICT_NUMBER"]],
 					schools=sgPlot.iter[["SCHOOL_NUMBER"]],
 					reports.by.student=sgPlot.reports.by.student,
+					reports.by.instructor=sgPlot.reports.by.instructor,
+					reports.by.school=sgPlot.reports.by.school,
 					sgPlot.years=tmp.years,
 					sgPlot.folder=sgPlot.folder,
 					sgPlot.folder.names=sgPlot.folder.names,
@@ -801,7 +941,9 @@ if (sgPlot.produce.plots) {
 					sgPlot.header.footer.color=sgPlot.header.footer.color,
 					sgPlot.fan=sgPlot.fan,
 					sgPlot.cleanup=sgPlot.cleanup,
-					sgPlot.baseline=sgPlot.baseline), mc.cores=par.start$workers, mc.preschedule=FALSE)
+					sgPlot.baseline=sgPlot.baseline,
+					sgPlot.zip=sgPlot.zip,
+					sgPlot.output.format=sgPlot.output.format), mc.cores=par.start$workers, mc.preschedule=FALSE)
 		}  ### END if MULTICORE
 		
 		stopParallel(parallel.config, par.start)
